# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import json
import sys
from typing import Any

from PySide6.QtWidgets import QTreeView, QApplication, QHeaderView
from PySide6.QtCore import QAbstractItemModel, QModelIndex, QObject, Qt, QFileInfo


class TreeItem:
    """A Json item corresponding to a line in QTreeView"""

    def __init__(self, parent: "TreeItem" = None):
        self._parent = parent
        self._key = ""
        self._value = ""
        self._value_type = None
        self._children = []

    def appendChild(self, item: "TreeItem"):
        """Add item as a child"""
        self._children.append(item)

    def child(self, row: int) -> "TreeItem":
        """Return the child of the current item from the given row"""
        return self._children[row]

    def parent(self) -> "TreeItem":
        """Return the parent of the current item"""
        return self._parent

    def childCount(self) -> int:
        """Return the number of children of the current item"""
        return len(self._children)

    def row(self) -> int:
        """Return the row where the current item occupies in the parent"""
        return self._parent._children.index(self) if self._parent else 0

    @property
    def key(self) -> str:
        """Return the key name"""
        return self._key

    @key.setter
    def key(self, key: str):
        """Set key name of the current item"""
        self._key = key

    @property
    def value(self) -> str:
        """Return the value name of the current item"""
        return self._value

    @value.setter
    def value(self, value: str):
        """Set value name of the current item"""
        self._value = value

    @property
    def value_type(self):
        """Return the python type of the item's value."""
        return self._value_type

    @value_type.setter
    def value_type(self, value):
        """Set the python type of the item's value."""
        self._value_type = value

    @classmethod
    def load(
        cls, value: list | dict, parent: "TreeItem" = None, sort=True
    ) -> "TreeItem":
        """Create a 'root' TreeItem from a nested list or a nested dictonary

        Examples:
            with open("file.json") as file:
                data = json.dump(file)
                root = TreeItem.load(data)

        This method is a recursive function that calls itself.

        Returns:
            TreeItem: TreeItem
        """
        rootItem = TreeItem(parent)
        rootItem.key = "root"

        if isinstance(value, dict):
            items = sorted(value.items()) if sort else value.items()

            for key, value in items:
                child = cls.load(value, rootItem)
                child.key = key
                child.value_type = type(value)
                rootItem.appendChild(child)

        elif isinstance(value, list):
            for index, value in enumerate(value):
                child = cls.load(value, rootItem)
                child.key = index
                child.value_type = type(value)
                rootItem.appendChild(child)

        else:
            rootItem.value = value
            rootItem.value_type = type(value)

        return rootItem


class JsonModel(QAbstractItemModel):
    """ An editable model of Json data """

    def __init__(self, parent: QObject = None):
        super().__init__(parent)

        self._rootItem = TreeItem()
        self._headers = ("key", "value")

    def clear(self):
        """ Clear data from the model """
        self.load({})

    def load(self, document: dict):
        """Load model from a nested dictionary returned by json.loads()

        Arguments:
            document (dict): JSON-compatible dictionary
        """

        assert isinstance(
            document, (dict, list, tuple)
        ), "`document` must be of dict, list or tuple, " f"not {type(document)}"

        self.beginResetModel()

        self._rootItem = TreeItem.load(document)
        self._rootItem.value_type = type(document)

        self.endResetModel()

        return True

    def data(self, index: QModelIndex, role: Qt.ItemDataRole) -> Any:
        """Override from QAbstractItemModel

        Return data from a json item according index and role

        """
        if not index.isValid():
            return None

        item = index.internalPointer()

        if role == Qt.ItemDataRole.DisplayRole:
            if index.column() == 0:
                return item.key

            if index.column() == 1:
                return item.value

        elif role == Qt.ItemDataRole.EditRole:
            if index.column() == 1:
                return item.value

    def setData(self, index: QModelIndex, value: Any, role: Qt.ItemDataRole):
        """Override from QAbstractItemModel

        Set json item according index and role

        Args:
            index (QModelIndex)
            value (Any)
            role (Qt.ItemDataRole)

        """
        if role == Qt.ItemDataRole.EditRole:
            if index.column() == 1:
                item = index.internalPointer()
                item.value = str(value)

                self.dataChanged.emit(index, index, [Qt.ItemDataRole.EditRole])

                return True

        return False

    def headerData(
        self, section: int, orientation: Qt.Orientation, role: Qt.ItemDataRole
    ):
        """Override from QAbstractItemModel

        For the JsonModel, it returns only data for columns (orientation = Horizontal)

        """
        if role != Qt.ItemDataRole.DisplayRole:
            return None

        if orientation == Qt.Orientation.Horizontal:
            return self._headers[section]

    def index(self, row: int, column: int, parent=QModelIndex()) -> QModelIndex:
        """Override from QAbstractItemModel

        Return index according row, column and parent

        """
        if not self.hasIndex(row, column, parent):
            return QModelIndex()

        if not parent.isValid():
            parentItem = self._rootItem
        else:
            parentItem = parent.internalPointer()

        childItem = parentItem.child(row)
        if childItem:
            return self.createIndex(row, column, childItem)
        else:
            return QModelIndex()

    def parent(self, index: QModelIndex) -> QModelIndex:
        """Override from QAbstractItemModel

        Return parent index of index

        """

        if not index.isValid():
            return QModelIndex()

        childItem = index.internalPointer()
        parentItem = childItem.parent()

        if parentItem == self._rootItem:
            return QModelIndex()

        return self.createIndex(parentItem.row(), 0, parentItem)

    def rowCount(self, parent=QModelIndex()):
        """Override from QAbstractItemModel

        Return row count from parent index
        """
        if parent.column() > 0:
            return 0

        if not parent.isValid():
            parentItem = self._rootItem
        else:
            parentItem = parent.internalPointer()

        return parentItem.childCount()

    def columnCount(self, parent=QModelIndex()):
        """Override from QAbstractItemModel

        Return column number. For the model, it always return 2 columns
        """
        return 2

    def flags(self, index: QModelIndex) -> Qt.ItemFlags:
        """Override from QAbstractItemModel

        Return flags of index
        """
        flags = super(JsonModel, self).flags(index)

        if index.column() == 1:
            return Qt.ItemFlag.ItemIsEditable | flags
        else:
            return flags

    def to_json(self, item=None):

        if item is None:
            item = self._rootItem

        nchild = item.childCount()

        if item.value_type is dict:
            document = {}
            for i in range(nchild):
                ch = item.child(i)
                document[ch.key] = self.to_json(ch)
            return document

        elif item.value_type == list:
            document = []
            for i in range(nchild):
                ch = item.child(i)
                document.append(self.to_json(ch))
            return document

        else:
            return item.value


if __name__ == "__main__":

    app = QApplication(sys.argv)
    view = QTreeView()
    model = JsonModel()

    view.setModel(model)

    json_path = QFileInfo(__file__).absoluteDir().filePath("example.json")

    with open(json_path) as file:
        document = json.load(file)
        model.load(document)

    view.show()
    view.header().setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)
    view.setAlternatingRowColors(True)
    view.resize(500, 300)
    app.exec()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations
import sys

from PySide6.QtCore import QTime, QTimer, Slot
from PySide6.QtWidgets import QApplication, QLCDNumber


class DigitalClock(QLCDNumber):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setSegmentStyle(QLCDNumber.SegmentStyle.Filled)
        self.setDigitCount(8)

        self.timer = QTimer(self)
        self.timer.timeout.connect(self.show_time)
        self.timer.start(1000)

        self.show_time()

        self.setWindowTitle("Digital Clock")
        self.resize(250, 60)

    @Slot()
    def show_time(self):
        time = QTime.currentTime()
        text = time.toString("hh:mm:ss")

        # Blinking effect
        if (time.second() % 2) == 0:
            text = text.replace(":", " ")

        self.display(text)


if __name__ == "__main__":

    app = QApplication(sys.argv)
    clock = DigitalClock()
    clock.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test case for PYSIDE-1354: Ensure that slots are invoked from the receiver's
thread context when using derived classes (and thus, a global receiver).'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, QThread, QTimer, Slot
from helper.usesqapplication import UsesQApplication


class ReceiverBase(QObject):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.senderThread = None

    @Slot()
    def slot_function(self):
        self.senderThread = QThread.currentThread()


class Receiver(ReceiverBase):
    pass


class TestThread(QThread):
    def __init__(self, parent=None):
        super().__init__(parent)

    def run(self):
        pass


class SignalAcrossThreads(UsesQApplication):
    def setUp(self):
        UsesQApplication.setUp(self)
        self._timer_tick = 0
        self._timer = QTimer()
        self._timer.setInterval(20)
        self._timer.timeout.connect(self._control_test)
        self._worker_thread = TestThread()

    def tearDown(self):
        UsesQApplication.tearDown(self)

    @Slot()
    def _control_test(self):
        if self._timer_tick == 0:
            self._worker_thread.start()
        elif self._timer_tick == 1:
            self._worker_thread.wait()
        else:
            self._timer.stop()
            self.app.quit()
        self._timer_tick += 1

    def test(self):
        worker_thread_receiver = Receiver()
        worker_thread_receiver.moveToThread(self._worker_thread)
        self._worker_thread.started.connect(worker_thread_receiver.slot_function)

        main_thread = QThread.currentThread()
        main_thread_receiver = Receiver()
        self._worker_thread.started.connect(main_thread_receiver.slot_function)

        self._timer.start()
        self.app.exec()

        self.assertEqual(worker_thread_receiver.senderThread, self._worker_thread)
        self.assertEqual(main_thread_receiver.senderThread, main_thread)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import Qt
from PySide6.QtGui import QKeySequence, qt_set_sequence_auto_mnemonic

from helper.usesqapplication import UsesQApplication


class QKeySequenceTest(UsesQApplication):

    def testGetItemOperator(self):
        # bug #774
        # PYSIDE-1735: Remapped from Qt.Modifier to Qt.KeyboardModifier
        #              Note that Qt.(Keyboard)?Modifier will be no longer IntFlag.
        ks = QKeySequence(Qt.KeyboardModifier.ShiftModifier, Qt.KeyboardModifier.ControlModifier,
                          Qt.Key.Key_P, Qt.Key.Key_R)
        self.assertEqual(ks[0].keyboardModifiers(), Qt.KeyboardModifier.ShiftModifier)
        self.assertEqual(ks[1].keyboardModifiers(), Qt.KeyboardModifier.ControlModifier)
        self.assertEqual(ks[2].key(), Qt.Key.Key_P)
        self.assertEqual(ks[3].key(), Qt.Key.Key_R)

        def testAutoMnemonic(self):
            qt_set_sequence_auto_mnemonic(True)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.helper import quickview_errorstring
from helper.timedqguiapplication import TimedQGuiApplication
from PySide6.QtCore import QObject, QUrl, Slot
from PySide6.QtQuick import QQuickView
from PySide6.QtQml import QmlElement

QML_IMPORT_NAME = "test.ProxyObject"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class ProxyObject(QObject):
    def __init__(self):
        super().__init__()
        self._o = None
        self._receivedName = ""

    @Slot(result='QObject*')
    def getObject(self):
        if self._o:
            return self._o

        self._o = QObject()
        self._o.setObjectName("PySideObject")
        return self._o

    @Slot(str)
    def receivedObject(self, name):
        self._receivedName = name


class TestConnectionWithInvalidSignature(TimedQGuiApplication):

    def testSlotRetur(self):
        view = QQuickView()
        proxy = ProxyObject()

        view.setInitialProperties({"proxy": proxy})
        file = Path(__file__).resolve().parent / 'bug_726.qml'
        self.assertTrue(file.is_file())
        view.setSource(QUrl.fromLocalFile(file))
        root = view.rootObject()
        self.assertTrue(root, quickview_errorstring(view))
        button = root.findChild(QObject, "buttonMouseArea")
        view.show()
        button.entered.emit()
        self.assertEqual(proxy._receivedName, "PySideObject")


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

from PySide6.QtCore import Signal, QObject


class SameNameSender(QObject):
    ''' Base class for the Test sender class of SameNameSenderTest (PYSIDE-3201).'''
    signal1 = Signal()
    signal2 = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test for Bug 630 - Fails to resolve overload for QCursor(QBitmap, QBitmap, int, int)
http://bugs.openbossa.org/show_bug.cgi?id=630
'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QBitmap, QCursor
from helper.usesqapplication import UsesQApplication


class TestQCursor(UsesQApplication):
    def testQCursorConstructor(self):
        bmp = QBitmap(16, 16)
        cursor = QCursor(bmp, bmp, 16, 16)  # noqa


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QRandomGenerator


class OverflowExceptionCollect(unittest.TestCase):
    '''Test case for OverflowError exception during garbage collection. See bug #147'''

    def assertRaises(self, *args, **kwds):
        if not hasattr(sys, "pypy_version_info"):
            # PYSIDE-535: PyPy complains "Fatal RPython error: NotImplementedError"
            return super().assertRaises(*args, **kwds)

    def testOverflow(self):
        # NOTE: PyQt4 raises TypeError, but boost.python raises OverflowError
        self.assertRaises(OverflowError, QRandomGenerator, 42415335332353253)
        # should not abort if bug #147 is fixed
        gc.collect()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test case for NumPy Array types.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
import sample

hasNumPy = False

try:
    import numpy
    hasNumPy = True
except ImportError:
    pass


class ArrayTester(unittest.TestCase):
    '''Test case for NumPy arrays.'''

    def testIntArray(self):
        intList = numpy.array([1, 2, 3, 4], dtype='int32')
        self.assertEqual(sample.sumIntArray(intList), 10)

    def testDoubleArray(self):
        doubleList = numpy.array([1, 2, 3, 4], dtype='double')
        self.assertEqual(sample.sumDoubleArray(doubleList), 10)

    def testIntMatrix(self):
        intMatrix = numpy.array([[1, 2, 3], [4, 5, 6]], dtype='int32')
        self.assertEqual(sample.sumIntMatrix(intMatrix), 21)

    def testDoubleMatrix(self):
        doubleMatrix = numpy.array([[1, 2, 3], [4, 5, 6]], dtype='double')
        self.assertEqual(sample.sumDoubleMatrix(doubleMatrix), 21)


if __name__ == '__main__' and hasNumPy:
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path
import sys

from PySide6.QtGui import QGuiApplication
from PySide6.QtCore import QCoreApplication
from PySide6.QtQml import QQmlDebuggingEnabler
from PySide6.QtQuick import QQuickView

from TextBalloon.textballoon import TextBalloon  # noqa: F401

if __name__ == "__main__":
    argument_parser = ArgumentParser(description="Scene Graph Painted Item Example",
                                     formatter_class=RawTextHelpFormatter)
    argument_parser.add_argument("-qmljsdebugger", action="store",
                                 help="Enable QML debugging")
    options = argument_parser.parse_args()
    if options.qmljsdebugger:
        QQmlDebuggingEnabler.enableDebugging(True)

    app = QGuiApplication(sys.argv)
    QCoreApplication.setOrganizationName("QtProject")
    QCoreApplication.setOrganizationDomain("qt-project.org")

    view = QQuickView()
    view.setResizeMode(QQuickView.ResizeMode.SizeRootObjectToView)
    view.engine().addImportPath(Path(__file__).parent)
    view.loadFromModule("painteditemexample", "Main")

    if view.status() == QQuickView.Status.Error:
        sys.exit(-1)
    view.show()

    exit_code = QCoreApplication.exec()
    del view
    sys.exit(exit_code)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import sys
import os

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths()

from PySide6.QtWidgets import QApplication, QLabel
from PySide6.QtCore import QObject, QTimer


class QtKeyPressListener(QObject):
    def __init__(self, obj):
        super().__init__()
        obj.installEventFilter(self)

    def eventFilter(self, obj, event):
        # This used to crash here due to a misbehaviour of type discovery!
        return QObject.eventFilter(self, obj, event)


app = QApplication([])
key_listener = QtKeyPressListener(app)
w = QLabel('Hello')
w.show()
QTimer.singleShot(0, w.close)
app.exec()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from enum import Enum, auto

from PySide6.QtCore import QFileInfo, QObject, QSettings, Signal, Slot


DEFAULT_MAX_FILES = 10


# Test if file exists and can be opened
def testFileAccess(fileName):
    return QFileInfo(fileName).isReadable()


class RemoveReason(Enum):
    Other = auto()
    Duplicate = auto()


class EmitPolicy(Enum):
    EmitWhenChanged = auto(),
    NeverEmit = auto()


s_maxFiles = "maxFiles"
s_openMode = "openMode"
s_fileNames = "fileNames"
s_file = "file"


class RecentFiles(QObject):

    countChanged = Signal(int)
    changed = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self._maxFiles = DEFAULT_MAX_FILES
        self._files = []

    # Access to QStringList member functions
    def recentFiles(self):
        return self._files

    def isEmpty(self):
        return not self._files

    # Properties
    def maxFiles(self):
        return self._maxFiles

    def setMaxFiles(self, maxFiles):
        self._maxFiles = maxFiles

    def addFile(self, fileName):
        self._addFile(fileName, EmitPolicy.EmitWhenChanged)

    def removeFile(self, fileName):
        idx = self._files.find(fileName)
        self._removeFile(idx, RemoveReason.Other)

    @Slot()
    def clear(self):
        if self.isEmpty():
            return
        self._files.clear()
        self.countChanged.emit(0)

    def _addFile(self, fileName, policy):
        if not testFileAccess(fileName):
            return

        # Remember size, as cleanup can result in a change without size change
        c = len(self._files)

        # Clean dangling and duplicate files
        i = 0
        while i < len(self._files):
            file = self._files[i]
            if not testFileAccess(file):
                self._removeFile(file, RemoveReason.Other)
            elif file == fileName:
                self._removeFile(file, RemoveReason.Duplicate)
            else:
                i += 1

        # Cut tail
        while len(self._files) > self._maxFiles:
            self.removeFile((len(self._files) - 1), RemoveReason.Other)

        self._files.insert(0, fileName)

        if policy == EmitPolicy.NeverEmit:
            return

        if policy == EmitPolicy.EmitWhenChanged:
            self.changed.emit()
            if c != len(self._files):
                self.countChanged.emit(len(self._files))

    @Slot(list)
    def addFiles(self, files):
        if files.isEmpty():
            return

        if len(files) == 1:
            self.addFile(files[0])
            return

        c = len(self._files)

        for file in files:
            self.addFile(file, EmitPolicy.NeverEmit)

        self.changed.emit()
        if len(self._files) != c:
            self.countChanged.emit(len(self._files))

    def _removeFile(self, p, reason):
        index = p
        if isinstance(p, str):
            index = self._files.index(p) if p in self._files else -1
        if index < 0 or index >= len(self._files):
            return
        del self._files[index]

        # No emit for duplicate removal, add emits changed later.
        if reason != RemoveReason.Duplicate:
            self.changed.emit()

    @Slot(QSettings, str)
    def saveSettings(self, settings, key):
        settings.beginGroup(key)
        settings.setValue(s_maxFiles, self.maxFiles())
        if self._files:
            settings.beginWriteArray(s_fileNames, len(self._files))
            for index, file in enumerate(self._files):
                settings.setArrayIndex(index)
                settings.setValue(s_file, file)
            settings.endArray()
        settings.endGroup()

    @Slot(QSettings, str)
    def restoreFromSettings(self, settings, key):
        settings.beginGroup(key)
        self.setMaxFiles(settings.value(s_maxFiles, DEFAULT_MAX_FILES, int))
        self._files.clear()  # clear list without emitting
        numberFiles = settings.beginReadArray(s_fileNames)
        for index in range(0, numberFiles):
            settings.setArrayIndex(index)
            absoluteFilePath = settings.value(s_file)
            self._addFile(absoluteFilePath, EmitPolicy.NeverEmit)
        settings.endArray()
        settings.endGroup()
        if self._files:
            self.changed.emit()
        return True



from PySide6.QtCore import QAbstractTableModel

COLS = 3
ROWS = 2


class MyModel(QAbstractTableModel):

    editCompleted = Signal(str)

    def __init__(self, parent=None):
        ...

    def rowCount(self, parent=None):
        ...

    def columnCount(self, parent=None):
        ...

    def data(self, index, role=Qt.DisplayRole):
        ...

    def setData(self, index, value, role):
        ...

    def flags(self, index):
        ...



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWidgets import QDialog, QLineEdit, QVBoxLayout

# Python binding from the C++ widget
from wiggly import WigglyWidget as WigglyWidgetCPP

# Python-only widget
from wigglywidget import WigglyWidget as WigglyWidgetPY


class Dialog(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        wiggly_widget_py = WigglyWidgetPY(self)
        wiggly_widget_cpp = WigglyWidgetCPP(self)
        lineEdit = QLineEdit(self)

        layout = QVBoxLayout(self)
        layout.addWidget(wiggly_widget_py)
        layout.addWidget(wiggly_widget_cpp)
        layout.addWidget(lineEdit)
        lineEdit.setClearButtonEnabled(True)
        wiggly_widget_py.running = True
        wiggly_widget_cpp.setRunning(True)

        lineEdit.textChanged.connect(wiggly_widget_py.setText)
        lineEdit.textChanged.connect(wiggly_widget_cpp.setText)
        lineEdit.setText("🖖 Hello world!")

        self.setWindowTitle("Wiggly")
        self.resize(360, 145)



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import QPoint, Qt, Slot
from PySide6.QtGui import QLinearGradient, QVector3D
from PySide6.QtGraphs import (QSurface3DSeries, QSurfaceDataItem,
                              QGraphsTheme)


DARK_RED_POS = 1.0
RED_POS = 0.8
YELLOW_POS = 0.6
GREEN_POS = 0.4
DARK_GREEN_POS = 0.2


class HighlightSeries(QSurface3DSeries):

    def __init__(self):
        super().__init__()
        self._width = 100
        self._height = 100
        self._srcWidth = 0
        self._srcHeight = 0
        self._position = QPoint()
        self._topographicSeries = None
        self._minHeight = 0.0
        self._height_adjustment = 5.0
        self.setDrawMode(QSurface3DSeries.DrawFlag.DrawSurface)
        self.setShading(QSurface3DSeries.Shading.Flat)
        self.setVisible(False)

    def setTopographicSeries(self, series):
        self._topographicSeries = series
        array = self._topographicSeries.dataArray()
        self._srcWidth = len(array[0])
        self._srcHeight = len(array)
        self._topographicSeries.selectedPointChanged.connect(self.handlePositionChange)

    def setMinHeight(self, height):
        self. m_minHeight = height

    @Slot(QPoint)
    def handlePositionChange(self, position):
        self._position = position

        if position == self.invalidSelectionPosition():
            self.setVisible(False)
            return

        halfWidth = self._width / 2
        halfHeight = self._height / 2

        startX = position.x() - halfWidth
        if startX < 0:
            startX = 0
        endX = position.x() + halfWidth
        if endX > (self._srcWidth - 1):
            endX = self._srcWidth - 1
        startZ = position.y() - halfHeight
        if startZ < 0:
            startZ = 0
        endZ = position.y() + halfHeight
        if endZ > (self._srcHeight - 1):
            endZ = self._srcHeight - 1

        srcArray = self._topographicSeries.dataArray()

        dataArray = []
        for i in range(int(startZ), int(endZ)):
            newRow = []
            srcRow = srcArray[i]
            for j in range(int(startX), int(endX)):
                pos = QVector3D(srcRow[j].position())
                pos.setY(pos.y() + self._height_adjustment)
                item = QSurfaceDataItem(pos)
                newRow.append(item)
            dataArray.append(newRow)
        self.dataProxy().resetArray(dataArray)
        self.setVisible(True)

    @Slot(float)
    def handleGradientChange(self, value):
        ratio = self._minHeight / value

        gr = QLinearGradient()
        gr.setColorAt(0.0, Qt.GlobalColor.black)
        gr.setColorAt(DARK_GREEN_POS * ratio, Qt.GlobalColor.darkGreen)
        gr.setColorAt(GREEN_POS * ratio, Qt.GlobalColor.green)
        gr.setColorAt(YELLOW_POS * ratio, Qt.GlobalColor.yellow)
        gr.setColorAt(RED_POS * ratio, Qt.GlobalColor.red)
        gr.setColorAt(DARK_RED_POS * ratio, Qt.GlobalColor.darkRed)

        self.setBaseGradient(gr)
        self.setColorStyle(QGraphsTheme.ColorStyle.RangeGradient)

        self.handle_zoom_change(ratio)

    def handle_zoom_change(self, zoom):
        self._height_adjustment = (1.2 - zoom) * 10.0



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from helper.helper import quickview_errorstring  # noqa: E402

from PySide6.QtCore import Property, QTimer, QUrl  # noqa: E402
from PySide6.QtGui import QGuiApplication, QPen, QColor, QPainter  # noqa: E402
from PySide6.QtQml import (qjsEngine, qmlContext, qmlEngine,  # noqa: E402
                           ListProperty, QmlElement, QmlNamedElement)
from PySide6.QtQuick import QQuickView, QQuickItem, QQuickPaintedItem  # noqa: E402


QML_IMPORT_NAME = "Charts"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class PieSlice (QQuickPaintedItem):
    def __init__(self, parent=None):
        QQuickPaintedItem.__init__(self, parent)
        self._color = QColor()
        self._fromAngle = 0
        self._angleSpan = 0

    def getColor(self):
        return self._color

    def setColor(self, value):
        self._color = value

    def getFromAngle(self):
        return self._angle

    def setFromAngle(self, value):
        self._fromAngle = value

    def getAngleSpan(self):
        return self._angleSpan

    def setAngleSpan(self, value):
        self._angleSpan = value

    color = Property(QColor, getColor, setColor)
    fromAngle = Property(int, getFromAngle, setFromAngle)
    angleSpan = Property(int, getAngleSpan, setAngleSpan)

    def paint(self, painter):
        global paintCalled
        pen = QPen(self._color, 2)
        painter.setPen(pen)
        painter.setRenderHints(QPainter.RenderHint.Antialiasing, True)
        painter.drawPie(self.boundingRect(), self._fromAngle * 16, self._angleSpan * 16)
        paintCalled = True


@QmlNamedElement("PieChart")
class PieChartOriginalName(QQuickItem):
    def __init__(self, parent=None):
        QQuickItem.__init__(self, parent)
        self._name = ''
        self._slices = []

    def getName(self):
        return self._name

    def setName(self, value):
        self._name = value

    name = Property(str, getName, setName)

    def appendSlice(self, _slice):
        global appendCalled
        _slice.setParentItem(self)
        self._slices.append(_slice)
        appendCalled = True

    slices = ListProperty(PieSlice, append=appendSlice)


appendCalled = False
paintCalled = False


class TestQmlSupport(unittest.TestCase):

    def testIt(self):
        app = QGuiApplication([])

        view = QQuickView()
        file = Path(__file__).resolve().parent / 'registertype.qml'
        self.assertTrue(file.is_file())
        view.setSource(QUrl.fromLocalFile(file))
        root_object = view.rootObject()
        self.assertTrue(root_object, quickview_errorstring(view))
        self.assertTrue(qjsEngine(root_object))
        self.assertEqual(qmlEngine(root_object), view.engine())
        self.assertTrue(qmlContext(root_object))

        view.show()
        QTimer.singleShot(250, view.close)
        app.exec()
        self.assertTrue(appendCalled)
        self.assertTrue(paintCalled)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Point


class MetaA(type):
    pass


class A:
    __metaclass__ = MetaA


MetaB = type(Point)
B = Point


class MetaC(MetaA, MetaB):
    pass


class C(A, B):
    __metaclass__ = MetaC


class D(C):
    pass


class TestMetaClass(unittest.TestCase):
    def testIt(self):
        w1 = C()  # works
        w1.setX(1)
        w1.setY(2)

        w2 = D()  # should work!
        w2.setX(3)
        w2.setY(4)

        w3 = w1 + w2
        self.assertEqual(w3.x(), 4)
        self.assertEqual(w3.y(), 6)



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QCoreApplication, QObject, SIGNAL, QTimer
from PySide6.QtStateMachine import QFinalState, QState, QStateMachine, QSignalTransition


class QStateTest(unittest.TestCase):
    def testBasic(self):
        app = QCoreApplication([])

        o = QObject()
        o.setProperty("text", "INdT")

        machine = QStateMachine()
        s1 = QState()
        s1.assignProperty(o, "text", "Rocks")

        s2 = QFinalState()
        t = s1.addTransition(o, SIGNAL("change()"), s2)
        self.assertTrue(isinstance(t, QSignalTransition))

        machine.addState(s1)
        machine.addState(s2)
        machine.setInitialState(s1)
        machine.start()

        o.emit(SIGNAL("change()"))

        QTimer.singleShot(100, app.quit)
        app.exec()

        txt = o.property("text")
        self.assertTrue(txt, "Rocks")


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import math
import sys

from PySide6.QtCore import (QLineF, QPointF, QRandomGenerator, QRectF, QTimer, Qt)
from PySide6.QtGui import (QBrush, QColor, QPainter, QPainterPath, QPixmap, QPolygonF, QTransform)
from PySide6.QtWidgets import (QApplication, QGraphicsItem, QGraphicsScene, QGraphicsView)

import mice_rc  # noqa: F401


def random(boundary):
    return QRandomGenerator.global_().bounded(boundary)


class Mouse(QGraphicsItem):
    PI = math.pi
    TWO_PI = 2.0 * PI

    # Create the bounding rectangle once.
    adjust = 0.5
    BOUNDING_RECT = QRectF(-20 - adjust, -22 - adjust, 40 + adjust,
                           83 + adjust)

    def __init__(self):
        super().__init__()

        self.angle = 0.0
        self.speed = 0.0
        self._mouse_eye_direction = 0.0
        self.color = QColor(random(256), random(256), random(256))

        self.setTransform(QTransform().rotate(random(360 * 16)))

    @staticmethod
    def normalize_angle(angle):
        while angle < 0:
            angle += Mouse.TWO_PI
        while angle > Mouse.TWO_PI:
            angle -= Mouse.TWO_PI
        return angle

    def boundingRect(self):
        return Mouse.BOUNDING_RECT

    def shape(self):
        path = QPainterPath()
        path.addRect(-10, -20, 20, 40)
        return path

    def paint(self, painter, option, widget):
        # Body.
        painter.setBrush(self.color)
        painter.drawEllipse(-10, -20, 20, 40)

        # Eyes.
        painter.setBrush(Qt.GlobalColor.white)
        painter.drawEllipse(-10, -17, 8, 8)
        painter.drawEllipse(2, -17, 8, 8)

        # Nose.
        painter.setBrush(Qt.GlobalColor.black)
        painter.drawEllipse(QRectF(-2, -22, 4, 4))

        # Pupils.
        painter.drawEllipse(QRectF(-8.0 + self._mouse_eye_direction, -17, 4, 4))
        painter.drawEllipse(QRectF(4.0 + self._mouse_eye_direction, -17, 4, 4))

        # Ears.
        if self.scene().collidingItems(self):
            painter.setBrush(Qt.GlobalColor.red)
        else:
            painter.setBrush(Qt.GlobalColor.darkYellow)

        painter.drawEllipse(-17, -12, 16, 16)
        painter.drawEllipse(1, -12, 16, 16)

        # Tail.
        path = QPainterPath(QPointF(0, 20))
        path.cubicTo(-5, 22, -5, 22, 0, 25)
        path.cubicTo(5, 27, 5, 32, 0, 30)
        path.cubicTo(-5, 32, -5, 42, 0, 35)
        painter.setBrush(Qt.BrushStyle.NoBrush)
        painter.drawPath(path)

    def advance(self, phase):
        if not phase:
            return
        # Don't move too far away.
        line_to_center = QLineF(QPointF(0, 0), self.mapFromScene(0, 0))
        if line_to_center.length() > 150:
            angle_to_center = math.acos(line_to_center.dx() / line_to_center.length())
            if line_to_center.dy() < 0:
                angle_to_center = Mouse.TWO_PI - angle_to_center
            angle_to_center = Mouse.normalize_angle((Mouse.PI - angle_to_center) + Mouse.PI / 2)

            if angle_to_center < Mouse.PI and angle_to_center > Mouse.PI / 4:
                # Rotate left.
                self.angle += [-0.25, 0.25][self.angle < -Mouse.PI / 2]
            elif (angle_to_center >= Mouse.PI
                    and angle_to_center < (Mouse.PI + Mouse.PI / 2 + Mouse.PI / 4)):
                # Rotate right.
                self.angle += [-0.25, 0.25][self.angle < Mouse.PI / 2]
        elif math.sin(self.angle) < 0:
            self.angle += 0.25
        elif math.sin(self.angle) > 0:
            self.angle -= 0.25

        # Try not to crash with any other mice.
        danger_mice = self.scene().items(QPolygonF([self.mapToScene(0, 0),
                                                    self.mapToScene(-30, -50),
                                                    self.mapToScene(30, -50)]))

        for item in danger_mice:
            if item is self:
                continue

            line_to_mouse = QLineF(QPointF(0, 0), self.mapFromItem(item, 0, 0))
            angle_to_mouse = math.acos(line_to_mouse.dx() / line_to_mouse.length())
            if line_to_mouse.dy() < 0:
                angle_to_mouse = Mouse.TWO_PI - angle_to_mouse
            angle_to_mouse = Mouse.normalize_angle((Mouse.PI - angle_to_mouse) + Mouse.PI / 2)

            if angle_to_mouse >= 0 and angle_to_mouse < Mouse.PI / 2:
                # Rotate right.
                self.angle += 0.5
            elif angle_to_mouse <= Mouse.TWO_PI and angle_to_mouse > (Mouse.TWO_PI - Mouse.PI / 2):
                # Rotate left.
                self.angle -= 0.5

        # Add some random movement.
        if len(danger_mice) > 1 and random(10) == 0:
            if random(2) != 0:
                self.angle += random(100) / 500.0
            else:
                self.angle -= random(100) / 500.0

        self.speed += (-50 + random(100)) / 100.0

        dx = math.sin(self.angle) * 10

        self._mouse_eye_direction = [dx / 5, 0.0][abs(dx / 5) < 1]

        self.setRotation(self.rotation() + dx)
        self.setPos(self.mapToParent(0, -(3 + math.sin(self.speed) * 3)))


if __name__ == '__main__':
    MOUSE_COUNT = 7
    app = QApplication(sys.argv)

    scene = QGraphicsScene()
    scene.setSceneRect(-300, -300, 600, 600)
    scene.setItemIndexMethod(QGraphicsScene.ItemIndexMethod.NoIndex)

    for i in range(MOUSE_COUNT):
        mouse = Mouse()
        mouse.setPos(math.sin((i * 6.28) / MOUSE_COUNT) * 200,
                     math.cos((i * 6.28) / MOUSE_COUNT) * 200)
        scene.addItem(mouse)

    view = QGraphicsView(scene)
    view.setRenderHint(QPainter.RenderHint.Antialiasing)
    view.setBackgroundBrush(QBrush(QPixmap(':/images/cheese.jpg')))
    view.setCacheMode(QGraphicsView.CacheModeFlag.CacheBackground)
    view.setViewportUpdateMode(QGraphicsView.ViewportUpdateMode.BoundingRectViewportUpdate)
    view.setDragMode(QGraphicsView.DragMode.ScrollHandDrag)
    view.setWindowTitle("Colliding Mice")
    view.resize(400, 300)
    view.show()

    timer = QTimer()
    timer.timeout.connect(scene.advance)
    timer.start(1000 / 33)
    sys.exit(app.exec())



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QBuffer
from PySide6.QtSvg import QSvgGenerator


REF_COUNT_DELTA = 2 if sys.version_info >= (3, 14) else 1


class QSvgGeneratorTest(unittest.TestCase):

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRefCountOfTOutputDevice(self):
        generator = QSvgGenerator()
        iodevice1 = QBuffer()
        refcount1 = sys.getrefcount(iodevice1)

        generator.setOutputDevice(iodevice1)

        self.assertEqual(generator.outputDevice(), iodevice1)
        self.assertEqual(sys.getrefcount(generator.outputDevice()),
                         refcount1 + REF_COUNT_DELTA)

        iodevice2 = QBuffer()
        refcount2 = sys.getrefcount(iodevice2)

        generator.setOutputDevice(iodevice2)

        self.assertEqual(generator.outputDevice(), iodevice2)
        self.assertEqual(sys.getrefcount(generator.outputDevice()),
                         refcount2 + REF_COUNT_DELTA)
        self.assertEqual(sys.getrefcount(iodevice1), refcount1)

        del generator

        self.assertEqual(sys.getrefcount(iodevice2), refcount2)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import subprocess
import sys
import xml.etree.ElementTree as ET
from pathlib import Path

from . import (QTPATHS_CMD, PYPROJECT_JSON_PATTERN, PYPROJECT_TOML_PATTERN, PYPROJECT_FILE_PATTERNS,
               ClOptions)
from .pyproject_toml import parse_pyproject_toml
from .pyproject_json import parse_pyproject_json


def run_command(command: list[str], cwd: str = None, ignore_fail: bool = False) -> int:
    """
    Run a command using a subprocess.
    If dry run is enabled, the command will be printed to stdout instead of being executed.

    :param command: The command to run including the arguments
    :param cwd: The working directory to run the command in
    :param ignore_fail: If True, the current process will not exit if the command fails

    :return: The exit code of the command
    """
    cloptions = ClOptions()
    if not cloptions.quiet or cloptions.dry_run:
        print(" ".join(command))
    if cloptions.dry_run:
        return 0

    ex = subprocess.call(command, cwd=cwd)
    if ex != 0 and not ignore_fail:
        sys.exit(ex)
    return ex


def qrc_file_requires_rebuild(resources_file_path: Path, compiled_resources_path: Path) -> bool:
    """Returns whether a compiled qrc file needs to be rebuilt based on the files that references"""
    root_element = ET.parse(resources_file_path).getroot()
    project_root = resources_file_path.parent

    files = [project_root / file.text for file in root_element.findall(".//file")]

    compiled_resources_time = compiled_resources_path.stat().st_mtime
    # If any of the resource files has been modified after the compiled qrc file, the compiled qrc
    # file needs to be rebuilt
    if any(file.is_file() and file.stat().st_mtime > compiled_resources_time for file in files):
        return True
    return False


def requires_rebuild(sources: list[Path], artifact: Path) -> bool:
    """Returns whether artifact needs to be rebuilt depending on sources"""
    if not artifact.is_file():
        return True

    artifact_mod_time = artifact.stat().st_mtime
    for source in sources:
        if source.stat().st_mtime > artifact_mod_time:
            return True
        # The .qrc file references other files that might have changed
        if source.suffix == ".qrc" and qrc_file_requires_rebuild(source, artifact):
            return True
    return False


def _remove_path_recursion(path: Path):
    """Recursion to remove a file or directory."""
    if path.is_file():
        path.unlink()
    elif path.is_dir():
        for item in path.iterdir():
            _remove_path_recursion(item)
        path.rmdir()


def remove_path(path: Path):
    """Remove path (file or directory) observing opt_dry_run."""
    cloptions = ClOptions()
    if not path.exists():
        return
    if not cloptions.quiet:
        print(f"Removing {path.name}...")
    if cloptions.dry_run:
        return
    _remove_path_recursion(path)


def package_dir() -> Path:
    """Return the PySide6 root."""
    return Path(__file__).resolve().parents[2]


_qtpaths_info: dict[str, str] = {}


def qtpaths() -> dict[str, str]:
    """Run qtpaths and return a dict of values."""
    global _qtpaths_info
    if not _qtpaths_info:
        output = subprocess.check_output([QTPATHS_CMD, "--query"])
        for line in output.decode("utf-8").split("\n"):
            tokens = line.strip().split(":", maxsplit=1)  # "Path=C:\..."
            if len(tokens) == 2:
                _qtpaths_info[tokens[0]] = tokens[1]
    return _qtpaths_info


_qt_metatype_json_dir: Path | None = None


def qt_metatype_json_dir() -> Path:
    """Return the location of the Qt QML metatype files."""
    global _qt_metatype_json_dir
    if not _qt_metatype_json_dir:
        qt_dir = package_dir()
        if sys.platform != "win32":
            qt_dir /= "Qt"
        metatypes_dir = qt_dir / "metatypes"
        if metatypes_dir.is_dir():  # Fully installed case
            _qt_metatype_json_dir = metatypes_dir
        else:
            # Fallback for distro builds/development.
            print(
                f"Falling back to {QTPATHS_CMD} to determine metatypes directory.", file=sys.stderr
            )
            _qt_metatype_json_dir = Path(qtpaths()["QT_INSTALL_ARCHDATA"]) / "metatypes"
    return _qt_metatype_json_dir


def resolve_valid_project_file(
    project_path_input: str = None, project_file_patterns: list[str] = PYPROJECT_FILE_PATTERNS
) -> Path:
    """
    Find a valid project file given a preferred project file name and a list of project file name
    patterns for a fallback search.

    If the provided file name is a valid project file, return it. Otherwise, search for a known
    project file in the current working directory with the given patterns.

    Raises a ValueError if no project file is found, multiple project files are found in the same
    directory or the provided path is not a valid project file or folder.

    :param project_path_input: The command-line argument specifying a project file or folder path.
    :param project_file_patterns: The list of project file patterns to search for.

    :return: The resolved project file path
    """
    if project_path_input and (project_file := Path(project_path_input).resolve()).is_file():
        if project_file.match(PYPROJECT_TOML_PATTERN):
            if bool(parse_pyproject_toml(project_file).errors):
                raise ValueError(f"Invalid project file: {project_file}")
        elif project_file.match(PYPROJECT_JSON_PATTERN):
            pyproject_json_result = parse_pyproject_json(project_file)
            if errors := '\n'.join(str(e) for e in pyproject_json_result.errors):
                raise ValueError(f"Invalid project file: {project_file}\n{errors}")
        else:
            raise ValueError(f"Unknown project file: {project_file}")
        return project_file

    project_folder = Path.cwd()
    if project_path_input:
        if not Path(project_path_input).resolve().is_dir():
            raise ValueError(f"Invalid project path: {project_path_input}")
        project_folder = Path(project_path_input).resolve()

    # Search a project file in the project folder using the provided patterns
    for pattern in project_file_patterns:
        if not (matches := list(project_folder.glob(pattern))):
            # No project files found with the specified pattern
            continue

        if len(matches) > 1:
            matched_files = '\n'.join(str(f) for f in matches)
            raise ValueError(f"Multiple project files found:\n{matched_files}")

        project_file = matches[0]

        if pattern == PYPROJECT_TOML_PATTERN:
            if parse_pyproject_toml(project_file).errors:
                # Invalid file, but a .pyproject file may exist
                # We can not raise an error due to ensuring backward compatibility
                continue
        elif pattern == PYPROJECT_JSON_PATTERN:
            pyproject_json_result = parse_pyproject_json(project_file)
            if errors := '\n'.join(str(e) for e in pyproject_json_result.errors):
                raise ValueError(f"Invalid project file: {project_file}\n{errors}")

        # Found a valid project file
        return project_file

    raise ValueError("No project file found in the current directory")



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject


def setValue(o):
    values = ['Name']
    o.setProperty('test1', values)


class QQtVersionTest(unittest.TestCase):
    '''Tests for QVariant conversion of QStringList'''

    def testGet(self):
        o = QObject()
        setValue(o)
        self.assertEqual(o.property('test1'), ['Name'])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit tests for QCommandLineParser and QCommandLineOption'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QCoreApplication, QCommandLineOption, QCommandLineParser


class QCommandLineParserTest(unittest.TestCase):
    def testParser(self):
        app = QCoreApplication([])  # noqa

        parser1 = QCommandLineParser()
        self.assertEqual(parser1.parse(["QtCore_qcommandlineparser_test", "file.txt"]), True)
        self.assertEqual(parser1.positionalArguments(), ["file.txt"])

        parser2 = QCommandLineParser()
        self.assertEqual(parser2.addOption(QCommandLineOption("b")), True)
        self.assertEqual(parser2.parse(["QtCore_qcommandlineparser_test", "-b"]), True)
        self.assertEqual(parser2.optionNames(), ["b"])
        self.assertEqual(parser2.isSet("b"), True)
        self.assertEqual(parser2.values("b"), [])
        self.assertEqual(parser2.positionalArguments(), [])


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from enum import IntEnum

from PySide6.QtCore import QUrl, Slot
from PySide6.QtGui import QStandardItemModel, QStandardItem
from PySide6.QtWidgets import QMainWindow
from PySide6.QtSerialBus import (QModbusDataUnit, QModbusDevice,
                                 QModbusRtuSerialClient, QModbusTcpClient)

from ui_mainwindow import Ui_MainWindow
from settingsdialog import SettingsDialog
from writeregistermodel import WriteRegisterModel


class ModbusConnection(IntEnum):
    SERIAL = 0
    TCP = 1


class MainWindow(QMainWindow):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self._modbus_device = None

        self._settings_dialog = SettingsDialog(self)

        self.init_actions()

        self._write_model = WriteRegisterModel(self)
        self._write_model.set_start_address(self.ui.writeAddress.value())
        self._write_model.set_number_of_values(self.ui.writeSize.currentText())

        self.ui.writeValueTable.setModel(self._write_model)
        self.ui.writeValueTable.hideColumn(2)
        vp = self.ui.writeValueTable.viewport()
        self._write_model.update_viewport.connect(vp.update)

        self.ui.writeTable.addItem("Coils", QModbusDataUnit.RegisterType.Coils)
        self.ui.writeTable.addItem("Discrete Inputs", QModbusDataUnit.RegisterType.DiscreteInputs)
        self.ui.writeTable.addItem("Input Registers", QModbusDataUnit.RegisterType.InputRegisters)
        self.ui.writeTable.addItem("Holding Registers",
                                   QModbusDataUnit.RegisterType.HoldingRegisters)

        self.ui.connectType.setCurrentIndex(0)
        self.onConnectTypeChanged(0)

        self._write_size_model = QStandardItemModel(0, 1, self)
        for i in range(1, 11):
            self._write_size_model.appendRow(QStandardItem(f"{i}"))
        self.ui.writeSize.setModel(self._write_size_model)
        self.ui.writeSize.setCurrentText("10")
        self.ui.writeSize.currentTextChanged.connect(self._write_model.set_number_of_values)

        self.ui.writeAddress.valueChanged.connect(self._write_model.set_start_address)
        self.ui.writeAddress.valueChanged.connect(self._writeAddress)

    @Slot(int)
    def _writeAddress(self, i):
        last_possible_index = 0
        currentIndex = self.ui.writeSize.currentIndex()
        for ii in range(0, 10):
            if ii < (10 - i):
                last_possible_index = ii
                self._write_size_model.item(ii).setEnabled(True)
            else:
                self._write_size_model.item(ii).setEnabled(False)
        if currentIndex > last_possible_index:
            self.ui.writeSize.setCurrentIndex(last_possible_index)

    def _close_device(self):
        if self._modbus_device:
            self._modbus_device.disconnectDevice()
            del self._modbus_device
            self._modbus_device = None

    def closeEvent(self, event):
        self._close_device()
        event.accept()

    def init_actions(self):
        self.ui.actionConnect.setEnabled(True)
        self.ui.actionDisconnect.setEnabled(False)
        self.ui.actionExit.setEnabled(True)
        self.ui.actionOptions.setEnabled(True)

        self.ui.connectButton.clicked.connect(self.onConnectButtonClicked)
        self.ui.actionConnect.triggered.connect(self.onConnectButtonClicked)
        self.ui.actionDisconnect.triggered.connect(self.onConnectButtonClicked)
        self.ui.readButton.clicked.connect(self.onReadButtonClicked)
        self.ui.writeButton.clicked.connect(self.onWriteButtonClicked)
        self.ui.readWriteButton.clicked.connect(self.onReadWriteButtonClicked)
        self.ui.connectType.currentIndexChanged.connect(self.onConnectTypeChanged)
        self.ui.writeTable.currentIndexChanged.connect(self.onWriteTableChanged)

        self.ui.actionExit.triggered.connect(self.close)
        self.ui.actionOptions.triggered.connect(self._settings_dialog.show)

    @Slot(int)
    def onConnectTypeChanged(self, index):
        self._close_device()

        if index == ModbusConnection.SERIAL:
            self._modbus_device = QModbusRtuSerialClient(self)
        elif index == ModbusConnection.TCP:
            self._modbus_device = QModbusTcpClient(self)
            if not self.ui.portEdit.text():
                self.ui.portEdit.setText("127.0.0.1:50200")

        self._modbus_device.errorOccurred.connect(self._show_device_errorstring)

        if not self._modbus_device:
            self.ui.connectButton.setDisabled(True)
            message = "Could not create Modbus client."
            self.statusBar().showMessage(message, 5000)
        else:
            self._modbus_device.stateChanged.connect(self.onModbusStateChanged)

    @Slot()
    def _show_device_errorstring(self):
        self.statusBar().showMessage(self._modbus_device.errorString(), 5000)

    @Slot()
    def onConnectButtonClicked(self):
        if not self._modbus_device:
            return

        self.statusBar().clearMessage()
        md = self._modbus_device
        if md.state() != QModbusDevice.State.ConnectedState:
            settings = self._settings_dialog.settings()
            if self.ui.connectType.currentIndex() == ModbusConnection.SERIAL:
                md.setConnectionParameter(QModbusDevice.ConnectionParameter.SerialPortNameParameter,
                                          self.ui.portEdit.text())
                md.setConnectionParameter(QModbusDevice.ConnectionParameter.SerialParityParameter,
                                          settings.parity)
                md.setConnectionParameter(QModbusDevice.ConnectionParameter.SerialBaudRateParameter,
                                          settings.baud)
                md.setConnectionParameter(QModbusDevice.ConnectionParameter.SerialDataBitsParameter,
                                          settings.data_bits)
                md.setConnectionParameter(QModbusDevice.ConnectionParameter.SerialStopBitsParameter,
                                          settings.stop_bits)
            else:
                url = QUrl.fromUserInput(self.ui.portEdit.text())
                md.setConnectionParameter(QModbusDevice.ConnectionParameter.NetworkPortParameter,
                                          url.port())
                md.setConnectionParameter(QModbusDevice.ConnectionParameter.NetworkAddressParameter,
                                          url.host())

            md.setTimeout(settings.response_time)
            md.setNumberOfRetries(settings.number_of_retries)
            if not md.connectDevice():
                message = "Connect failed: " + md.errorString()
                self.statusBar().showMessage(message, 5000)
            else:
                self.ui.actionConnect.setEnabled(False)
                self.ui.actionDisconnect.setEnabled(True)

        else:
            md.disconnectDevice()
            self.ui.actionConnect.setEnabled(True)
            self.ui.actionDisconnect.setEnabled(False)

    @Slot(int)
    def onModbusStateChanged(self, state):
        connected = (state != QModbusDevice.State.UnconnectedState)
        self.ui.actionConnect.setEnabled(not connected)
        self.ui.actionDisconnect.setEnabled(connected)

        if state == QModbusDevice.State.UnconnectedState:
            self.ui.connectButton.setText("Connect")
        elif state == QModbusDevice.State.ConnectedState:
            self.ui.connectButton.setText("Disconnect")

    @Slot()
    def onReadButtonClicked(self):
        if not self._modbus_device:
            return
        self.ui.readValue.clear()
        self.statusBar().clearMessage()
        reply = self._modbus_device.sendReadRequest(self.read_request(),
                                                    self.ui.serverEdit.value())
        if reply:
            if not reply.isFinished():
                reply.finished.connect(self.onReadReady)
            else:
                del reply  # broadcast replies return immediately
        else:
            message = "Read error: " + self._modbus_device.errorString()
            self.statusBar().showMessage(message, 5000)

    @Slot()
    def onReadReady(self):
        reply = self.sender()
        if not reply:
            return

        if reply.error() == QModbusDevice.NoError:
            unit = reply.result()
            total = unit.valueCount()
            for i in range(0, total):
                addr = unit.startAddress() + i
                value = unit.value(i)
                if unit.registerType().value <= QModbusDataUnit.Coils.value:
                    entry = f"Address: {addr}, Value: {value}"
                else:
                    entry = f"Address: {addr}, Value: {value:x}"
                self.ui.readValue.addItem(entry)

        elif reply.error() == QModbusDevice.ProtocolError:
            e = reply.errorString()
            ex = reply.rawResult().exceptionCode()
            message = f"Read response error: {e} (Modbus exception: 0x{ex:x})"
            self.statusBar().showMessage(message, 5000)
        else:
            e = reply.errorString()
            code = int(reply.error())
            message = f"Read response error: {e} (code: 0x{code:x})"
            self.statusBar().showMessage(message, 5000)

        reply.deleteLater()

    @Slot()
    def onWriteButtonClicked(self):
        if not self._modbus_device:
            return
        self.statusBar().clearMessage()

        write_unit = self.write_request()
        total = write_unit.valueCount()
        table = write_unit.registerType()
        for i in range(0, total):
            addr = i + write_unit.startAddress()
            if table == QModbusDataUnit.RegisterType.Coils:
                write_unit.setValue(i, self._write_model.m_coils[addr])
            else:
                write_unit.setValue(i, self._write_model.m_holdingRegisters[addr])

        reply = self._modbus_device.sendWriteRequest(write_unit,
                                                     self.ui.serverEdit.value())
        if reply:
            if reply.isFinished():
                # broadcast replies return immediately
                reply.deleteLater()
            else:
                reply.finished.connect(self._write_finished)
        else:
            message = "Write error: " + self._modbus_device.errorString()
            self.statusBar().showMessage(message, 5000)

    @Slot()
    def _write_finished(self):
        reply = self.sender()
        if not reply:
            return
        error = reply.error()
        if error == QModbusDevice.ProtocolError:
            e = reply.errorString()
            ex = reply.rawResult().exceptionCode()
            message = f"Write response error: {e} (Modbus exception: 0x{ex:x}"
            self.statusBar().showMessage(message, 5000)
        elif error != QModbusDevice.NoError:
            e = reply.errorString()
            message = f"Write response error: {e} (code: 0x{error:x})"
            self.statusBar().showMessage(message, 5000)
        reply.deleteLater()

    @Slot()
    def onReadWriteButtonClicked(self):
        if not self._modbus_device:
            return
        self.ui.readValue.clear()
        self.statusBar().clearMessage()

        write_unit = self.write_request()
        table = write_unit.registerType()
        total = write_unit.valueCount()
        for i in range(0, total):
            addr = i + write_unit.startAddress()
            if table == QModbusDataUnit.Coils:
                write_unit.setValue(i, self._write_model.m_coils[addr])
            else:
                write_unit.setValue(i, self._write_model.m_holdingRegisters[addr])

        reply = self._modbus_device.sendReadWriteRequest(self.read_request(),
                                                         write_unit,
                                                         self.ui.serverEdit.value())
        if reply:
            if not reply.isFinished():
                reply.finished.connect(self.onReadReady)
            else:
                del reply  # broadcast replies return immediately
        else:
            message = "Read error: " + self._modbus_device.errorString()
            self.statusBar().showMessage(message, 5000)

    @Slot(int)
    def onWriteTableChanged(self, index):
        coils_or_holding = index == 0 or index == 3
        if coils_or_holding:
            self.ui.writeValueTable.setColumnHidden(1, index != 0)
            self.ui.writeValueTable.setColumnHidden(2, index != 3)
            self.ui.writeValueTable.resizeColumnToContents(0)

        self.ui.readWriteButton.setEnabled(index == 3)
        self.ui.writeButton.setEnabled(coils_or_holding)
        self.ui.writeGroupBox.setEnabled(coils_or_holding)

    def read_request(self):
        table = self.ui.writeTable.currentData()

        start_address = self.ui.readAddress.value()
        assert start_address >= 0 and start_address < 10

        # do not go beyond 10 entries
        number_of_entries = min(int(self.ui.readSize.currentText()),
                                10 - start_address)
        return QModbusDataUnit(table, start_address, number_of_entries)

    def write_request(self):
        table = self.ui.writeTable.currentData()

        start_address = self.ui.writeAddress.value()
        assert start_address >= 0 and start_address < 10

        # do not go beyond 10 entries
        number_of_entries = min(int(self.ui.writeSize.currentText()),
                                10 - start_address)
        return QModbusDataUnit(table, start_address, number_of_entries)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for snake case generation'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import (SnakeCaseTest, SnakeCaseDerivedTest,
                    snake_case_global_function)


class OverrideTest(SnakeCaseDerivedTest):
    def virtual_func(self):
        return 4711


class SnakeCaseTestCase(unittest.TestCase):
    '''Test for SnakeCaseTest'''
    def testMemberFunctions(self):
        s = SnakeCaseTest()
        self.assertEqual(s.test_function1(), 42)

        self.assertEqual(s.testFunctionDisabled(), 42)

        self.assertEqual(s.testFunctionBoth(), 42)
        self.assertEqual(s.test_function_both(), 42)

    def virtualFunctions(self):
        s = OverrideTest()
        self.assertEqual(s.call_virtual_func(), 4711)

    def testMemberFields(self):
        s = SnakeCaseTest()
        old_value = s.test_field
        s.test_field = old_value + 1
        self.assertEqual(s.test_field, old_value + 1)

        old_value = s.testFieldDisabled
        s.testFieldDisabled = old_value + 1
        self.assertEqual(s.testFieldDisabled, old_value + 1)

        old_value = s.test_field_both
        s.test_field_both = old_value + 1
        self.assertEqual(s.test_field_both, old_value + 1)
        self.assertEqual(s.testFieldBoth, old_value + 1)

    def testGlobalFunction(self):
        self.assertEqual(snake_case_global_function(), 42)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from math import sqrt

from PySide6.QtWidgets import (QListView, QTreeView)
from PySide6.QtGui import QIcon, QKeySequence, QPainter
from PySide6.QtCore import (QDir, QIODevice, QModelIndex,
                            QPointF, Slot)
from PySide6.QtPrintSupport import QPrinter
from PySide6.QtPdf import QPdfDocument, QPdfBookmarkModel
from PySide6.QtPdfWidgets import QPdfView, QPdfPageSelector

from abstractviewer import AbstractViewer
from pdfviewer.zoomselector import ZoomSelector


ZOOM_MULTIPLIER = sqrt(2.0)


class PdfViewer(AbstractViewer):

    def __init__(self):
        super().__init__()
        self.uiInitialized.connect(self.initPdfViewer)
        self._toolBar = None
        self._zoomSelector = None
        self._pageSelector = None
        self._document = None
        self._pdfView = None
        self._actionForward = None
        self._actionBack = None
        self._bookmarks = None
        self._pages = None

    def init(self, file, parent, mainWindow):
        self._pdfView = QPdfView(parent)
        super().init(file, self._pdfView, mainWindow)
        self._document = QPdfDocument(self)

    def supportedMimeTypes(self):
        return ["application/pdf"]

    def initPdfViewer(self):
        self._toolBar = self.addToolBar("PDF")
        self._zoomSelector = ZoomSelector(self._toolBar)

        nav = self._pdfView.pageNavigator()
        self._pageSelector = QPdfPageSelector(self._toolBar)
        self._toolBar.insertWidget(self._uiAssets_forward, self._pageSelector)
        self._pageSelector.setDocument(self._document)
        self._pageSelector.currentPageChanged.connect(self.pageSelected)
        nav.currentPageChanged.connect(self._pageSelector.setCurrentPage)
        nav.backAvailableChanged.connect(self._uiAssets_back.setEnabled)
        self._actionBack = self._uiAssets_back
        self._actionForward = self._uiAssets_forward
        self._uiAssets_back.triggered.connect(self.onActionBackTriggered)
        self._uiAssets_forward.triggered.connect(self.onActionForwardTriggered)

        self._toolBar.addSeparator()
        self._toolBar.addWidget(self._zoomSelector)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.ZoomIn,
                               QIcon(":/demos/documentviewer/images/zoom-in.png"))
        actionZoomIn = self._toolBar.addAction(icon, "Zoom in", QKeySequence.StandardKey.ZoomIn)
        actionZoomIn.setToolTip("Increase zoom level")
        actionZoomIn.triggered.connect(self.onActionZoomInTriggered)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.ZoomOut,
                               QIcon(":/demos/documentviewer/images/zoom-out.png"))
        actionZoomOut = self._toolBar.addAction(icon, "Zoom out", QKeySequence.StandardKey.ZoomOut)
        actionZoomOut.setToolTip("Decrease zoom level")
        actionZoomOut.triggered.connect(self.onActionZoomOutTriggered)

        nav.backAvailableChanged.connect(self._actionBack.setEnabled)
        nav.forwardAvailableChanged.connect(self._actionForward.setEnabled)

        self._zoomSelector.zoomModeChanged.connect(self._pdfView.setZoomMode)
        self._zoomSelector.zoomFactorChanged.connect(self._pdfView.setZoomFactor)
        self._zoomSelector.reset()

        bookmarkModel = QPdfBookmarkModel(self)
        bookmarkModel.setDocument(self._document)
        self._uiAssets_tabs.clear()
        self._bookmarks = QTreeView(self._uiAssets_tabs)
        self._bookmarks.activated.connect(self.bookmarkSelected)
        self._bookmarks.setModel(bookmarkModel)
        self._pdfView.setDocument(self._document)
        self._pdfView.setPageMode(QPdfView.PageMode.MultiPage)

        self.openPdfFile()
        if not self._document.pageCount():
            return

        self._pages = QListView(self._uiAssets_tabs)
        self._pages.setModel(self._document.pageModel())

        self._pages.selectionModel().currentRowChanged.connect(self._currentRowChanged)
        self._pdfView.pageNavigator().currentPageChanged.connect(self._pageChanged)

        self._uiAssets_tabs.addTab(self._pages, "Pages")
        self._uiAssets_tabs.addTab(self._bookmarks, "Bookmarks")

    def viewerName(self):
        return "PdfViewer"

    @Slot(QModelIndex, QModelIndex)
    def _currentRowChanged(self, current, previous):
        if previous == current:
            return

        nav = self._pdfView.pageNavigator()
        row = current.row()
        if nav.currentPage() == row:
            return
        nav.jump(row, QPointF(), nav.currentZoom())

    @Slot(int)
    def _pageChanged(self, page):
        if self._pages.currentIndex().row() == page:
            return
        self._pages.setCurrentIndex(self._pages.model().index(page, 0))

    @Slot()
    def openPdfFile(self):
        self.disablePrinting()

        if self._file.open(QIODevice.OpenModeFlag.ReadOnly):
            self._document.load(self._file)

        documentTitle = self._document.metaData(QPdfDocument.MetaDataField.Title)
        if not documentTitle:
            documentTitle = "PDF Viewer"
        self.statusMessage(documentTitle)
        self.pageSelected(0)

        file_name = QDir.toNativeSeparators(self._file.fileName())
        self.statusMessage(f"Opened PDF file {file_name}")
        self.maybeEnablePrinting()

    def hasContent(self):
        return self._document if self._document.pageCount() > 0 else False

    def supportsOverview(self):
        return True

    def printDocument(self, printer):
        if not self.hasContent():
            return

        painter = QPainter()
        painter.begin(printer)
        pageRect = printer.pageRect(QPrinter.Unit.DevicePixel).toRect()
        pageSize = pageRect.size()
        for i in range(0, self._document.pageCount()):
            if i > 0:
                printer.newPage()
            page = self._document.render(i, pageSize)
            painter.drawImage(pageRect, page)
        painter.end()

    @Slot(QModelIndex)
    def bookmarkSelected(self, index):
        if not index.isValid():
            return

        page = index.data(int(QPdfBookmarkModel.Role.Page))
        zoomLevel = index.data(int(QPdfBookmarkModel.Role.Level)).toReal()
        self._pdfView.pageNavigator().jump(page, QPointF(), zoomLevel)

    @Slot(int)
    def pageSelected(self, page):
        nav = self._pdfView.pageNavigator()
        nav.jump(page, QPointF(), nav.currentZoom())

    @Slot()
    def onActionZoomInTriggered(self):
        self._pdfView.setZoomFactor(self._pdfView.zoomFactor() * ZOOM_MULTIPLIER)

    @Slot()
    def onActionZoomOutTriggered(self):
        self._pdfView.setZoomFactor(self._pdfView.zoomFactor() / ZOOM_MULTIPLIER)

    @Slot()
    def onActionPreviousPageTriggered(self):
        nav = self._pdfView.pageNavigator()
        nav.jump(nav.currentPage() - 1, QPointF(), nav.currentZoom())

    @Slot()
    def onActionNextPageTriggered(self):
        nav = self._pdfView.pageNavigator()
        nav.jump(nav.currentPage() + 1, QPointF(), nav.currentZoom())

    @Slot()
    def onActionBackTriggered(self):
        self._pdfView.pageNavigator().back()

    @Slot()
    def onActionForwardTriggered(self):
        self._pdfView.pageNavigator().forward()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from helper.helper import quickview_errorstring  # noqa: E402

from PySide6.QtCore import QObject, QUrl, Slot, QTimer  # noqa: E402
from PySide6.QtGui import QGuiApplication  # noqa: E402
from PySide6.QtQml import QQmlIncubationController, VolatileBool  # noqa: E402
from PySide6.QtQuick import QQuickView  # noqa: E402


class CustomIncubationController(QObject, QQmlIncubationController):
    def __init__(self, test):
        QObject.__init__(self)
        QQmlIncubationController.__init__(self)
        self.test = test
        self.interrupted = False

        # Incubate every 50 milliseconds
        self.startTimer(50)
        self.incubationShouldContinue = VolatileBool(True)
        self.test.assertEqual(self.incubationShouldContinue.get(), True)

    @Slot()
    def interrupter(self):
        if not self.interrupted:
            self.interrupted = True
            self.incubationShouldContinue.set(False)
            self.test.assertEqual(self.incubationShouldContinue.get(), False)
            QTimer.singleShot(0, QGuiApplication.instance().quit)

    def timerEvent(self, ev):
        # Incubate items for 2000 milliseconds, or until the volatile bool is set to false.
        self.incubateWhile(self.incubationShouldContinue, 2000)


class TestBug(unittest.TestCase):
    def testIncubateWhileCall(self):
        app = QGuiApplication(sys.argv)
        view = QQuickView()
        controller = CustomIncubationController(self)
        view.engine().setIncubationController(controller)
        view.setResizeMode(QQuickView.ResizeMode.SizeRootObjectToView)
        file = Path(__file__).resolve().parent / 'qqmlincubator_incubateWhile.qml'
        self.assertTrue(file.is_file())
        view.setSource(QUrl.fromLocalFile(file))
        self.assertTrue(view.rootObject(), quickview_errorstring(view))
        view.show()

        root = view.rootObject()
        # The QML code will issue an interrupt signal after half of its items are loaded.
        root.shouldInterrupt.connect(controller.interrupter)
        res = app.exec()  # noqa: F841

        itemsToCreate = root.property("itemsToCreate")
        loadedItems = root.property("loadedItems")
        self.assertEqual(loadedItems, itemsToCreate / 2)

        # Finish incubating the remaining items.
        controller.incubateFor(1000)
        loadedItems = root.property("loadedItems")
        self.assertEqual(loadedItems, itemsToCreate)

        # Deleting the view before it goes out of scope is required to make sure all child QML
        # instances are destroyed in the correct order.
        del view
        del app
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations


from pathlib import Path
from PySide6.QtCore import (
    QEasingCurve,
    QPointF,
    Qt,
    QAbstractAnimation,
    QPropertyAnimation,
    Property,
)
from PySide6.QtGui import QPainter, QTransform, QPixmap
from PySide6.QtWidgets import QGraphicsView, QFrame, QGraphicsScene, QGraphicsPixmapItem
from math import pi, sin, cos
from blureffect import BlurEffect


class BlurPicker(QGraphicsView):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._index = 0
        self._animation = QPropertyAnimation(self, b"index")
        self._path = Path(__file__).resolve().parent

        self._background = QPixmap(self._path / "images" / "background.jpg")
        self.setBackgroundBrush(self._background)
        self.setScene(QGraphicsScene(self))

        self._icons = []

        self.setup_scene()
        self.index = 0

        self._animation.setDuration(400)
        self._animation.setEasingCurve(QEasingCurve.Type.InOutSine)

        self.setRenderHint(QPainter.RenderHint.Antialiasing, True)
        self.setFrameStyle(QFrame.Shape.NoFrame)

    @Property(float)
    def index(self) -> float:
        return self._index

    @index.setter
    def index(self, index: float):
        self._index = index

        base_line = 0.0
        iconAngle = 2 * pi / len(self._icons)

        for i, icon in enumerate(self._icons):
            a = (i + self._index) * iconAngle
            xs = 170 * sin(a)
            ys = 100 * cos(a)
            pos = QPointF(xs, ys)
            pos = QTransform().rotate(-20).map(pos)
            pos -= QPointF(40, 40)
            icon.setPos(pos)
            base_line = max(base_line, ys)

            icon.graphicsEffect().set_base_line(base_line)

        self.scene().update()

    def setup_scene(self):
        self.scene().setSceneRect(-200, -120, 400, 240)

        names = ["accessories-calculator.png", "accessories-text-editor.png",
                 "help-browser.png", "internet-group-chat.png",
                 "internet-mail.png", "internet-web-browser.png", "office-calendar.png",
                 "system-users.png"]

        for name in names:
            pixmap = QPixmap(self._path / "images" / name)
            icon: QGraphicsPixmapItem = self.scene().addPixmap(pixmap)
            icon.setZValue(1)
            icon.setGraphicsEffect(BlurEffect(icon))
            self._icons.append(icon)

        bg: QGraphicsPixmapItem = self.scene().addPixmap(self._background)
        bg.setZValue(0)
        bg.setPos(-200, -150)

    def keyPressEvent(self, event):
        delta = 0
        if event.key() == Qt.Key.Key_Left:
            delta = -1
        elif event.key() == Qt.Key.Key_Right:
            delta = 1

        if self._animation.state() == QAbstractAnimation.Stopped and delta:
            self._animation.setEndValue(self._index + delta)
            self._animation.start()
            event.accept()

    def mousePressEvent(self, event):
        right = event.position().x() > (self.width() / 2)
        delta = 1 if right else -1

        if self._animation.state() == QAbstractAnimation.State.Stopped:
            self._animation.setEndValue(self._index + delta)
            self._animation.start()
            event.accept()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from tictactoe import TicTacToe

from PySide6.QtCore import Slot
from PySide6.QtGui import QAction
from PySide6.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout
from PySide6.QtDesigner import (QExtensionFactory, QPyDesignerTaskMenuExtension)


class TicTacToeDialog(QDialog):
    def __init__(self, parent):
        super().__init__(parent)
        layout = QVBoxLayout(self)
        self._ticTacToe = TicTacToe(self)
        layout.addWidget(self._ticTacToe)
        button_box = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok
                                      | QDialogButtonBox.StandardButton.Cancel
                                      | QDialogButtonBox.StandardButton.Reset)
        button_box.accepted.connect(self.accept)
        button_box.rejected.connect(self.reject)
        reset_button = button_box.button(QDialogButtonBox.StandardButton.Reset)
        reset_button.clicked.connect(self._ticTacToe.clear_board)
        layout.addWidget(button_box)

    def set_state(self, new_state):
        self._ticTacToe.setState(new_state)

    def state(self):
        return self._ticTacToe.state


class TicTacToeTaskMenu(QPyDesignerTaskMenuExtension):
    def __init__(self, ticTacToe, parent):
        super().__init__(parent)
        self._ticTacToe = ticTacToe
        self._edit_state_action = QAction('Edit State...', None)
        self._edit_state_action.triggered.connect(self._edit_state)

    def taskActions(self):
        return [self._edit_state_action]

    def preferredEditAction(self):
        return self._edit_state_action

    @Slot()
    def _edit_state(self):
        dialog = TicTacToeDialog(self._ticTacToe)
        dialog.set_state(self._ticTacToe.state)
        if dialog.exec() == QDialog.DialogCode.Accepted:
            self._ticTacToe.state = dialog.state()


class TicTacToeTaskMenuFactory(QExtensionFactory):
    def __init__(self, extension_manager):
        super().__init__(extension_manager)

    @staticmethod
    def task_menu_iid():
        return 'org.qt-project.Qt.Designer.TaskMenu'

    def createExtension(self, object, iid, parent):
        if iid != TicTacToeTaskMenuFactory.task_menu_iid():
            return None
        if object.__class__.__name__ != 'TicTacToe':
            return None
        return TicTacToeTaskMenu(object, parent)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for renaming using target-lang-name attribute.'''

import os
import re
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import RenamedValue, RenamedUser

from shibokensupport.signature import get_signature


class RenamingTest(unittest.TestCase):
    def test(self):
        '''Tests whether the C++ class ToBeRenamedValue renamed via attribute
           target-lang-name to RenamedValue shows up in consuming function
           signature strings correctly.
        '''
        renamed_value = RenamedValue()
        self.assertEqual(str(type(renamed_value)),
                         "<class 'sample.RenamedValue'>")
        rename_user = RenamedUser()
        rename_user.useRenamedValue(renamed_value)
        actual_signature = str(get_signature(rename_user.useRenamedValue))
        self.assertTrue(re.match(r"^\(self,\s*?v:\s*?sample.RenamedValue(,\s*?/)?\)\s*?->\s*?None$",
                                 actual_signature))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QQuickView'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.helper import quickview_errorstring
from helper.timedqguiapplication import TimedQGuiApplication

from PySide6.QtCore import QUrl, QObject, Property, Slot, Signal
from PySide6.QtQml import QQmlEngine, QQmlContext
from PySide6.QtQuick import QQuickView


class MyObject(QObject):
    titleChanged = Signal()

    def __init__(self, text, parent=None):
        QObject.__init__(self, parent)
        self._text = text

    def getText(self):
        return self._text

    @Slot(str)
    def qmlText(self, text):
        self._qmlText = text

    title = Property(str, getText, notify=titleChanged)


class TestQQuickView(TimedQGuiApplication):

    def testQQuickViewList(self):
        view = QQuickView()

        dataList = ["Item 1", "Item 2", "Item 3", "Item 4"]

        view.setInitialProperties({"model": dataList})

        file = Path(__file__).resolve().parent / 'view.qml'
        self.assertTrue(file.is_file())
        url = QUrl.fromLocalFile(file)
        view.setSource(url)
        self.assertTrue(view.rootObject(), quickview_errorstring(view))
        view.show()

        self.assertEqual(view.status(), QQuickView.Status.Ready)
        rootObject = view.rootObject()
        self.assertTrue(rootObject)
        context = QQmlEngine.contextForObject(rootObject)
        self.assertTrue(context)
        self.assertTrue(context.engine())

        test_context = QQmlContext(context)  # Context properties, PYSIDE-1921
        prop_pair = QQmlContext.PropertyPair()
        prop_pair.name = "testProperty"
        prop_pair.value = 42
        test_context.setContextProperties([prop_pair])
        self.assertTrue(test_context.contextProperty("testProperty"), 42)

    def testModelExport(self):
        view = QQuickView()
        dataList = [MyObject("Item 1"), MyObject("Item 2"), MyObject("Item 3"), MyObject("Item 4")]

        view.setInitialProperties({"model": dataList})

        file = Path(__file__).resolve().parent / 'viewmodel.qml'
        self.assertTrue(file.is_file())
        url = QUrl.fromLocalFile(file)
        view.setSource(url)
        self.assertTrue(view.rootObject(), quickview_errorstring(view))
        view.show()

        self.assertEqual(view.status(), QQuickView.Status.Ready)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the widgets/effects/blurpicker example from Qt v6.x"""

import sys
from PySide6.QtWidgets import QApplication
from blurpicker import BlurPicker


if __name__ == "__main__":
    app = QApplication(sys.argv)

    blur_picker = BlurPicker()
    blur_picker.setWindowTitle("Application Picker")

    blur_picker.setFixedSize(400, 300)
    blur_picker.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtWidgets import (QApplication, QFileSystemModel, QLabel,
                               QVBoxLayout, QWidget)
from PySide6.QtGui import QPalette
from PySide6.QtCore import QDir, Qt


if __name__ == '__main__':
    app = QApplication(sys.argv)

    window = QWidget()
    layout = QVBoxLayout(window)
    title = QLabel("Some items from the directory model", window)
    title.setBackgroundRole(QPalette.Base)
    title.setMargin(8)
    layout.addWidget(title)

#! [0]
    model = QFileSystemModel()
    model.setRootPath(QDir.currentPath())

    def on_directory_loaded(directory):
        parent_index = model.index(directory)
        num_rows = model.rowCount(parent_index)
#! [1]
        for row in range(num_rows):
            index = model.index(row, 0, parent_index)
#! [1]
#! [2]
            text = model.data(index, Qt.DisplayRole)
#! [2]
            label = QLabel(text, window)
            layout.addWidget(label)

    model.directoryLoaded.connect(on_directory_loaded)
#! [0]

    window.setWindowTitle("A simple model example")
    window.show()
    sys.exit(app.exec())



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for the SmartPtrTester class'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from other import SmartPtrTester


class SmartPtrTest(unittest.TestCase):
    '''Test case for the SmartPtrTester class'''

    def test(self):
        tester = SmartPtrTester()

        integerPtr = tester.createSharedPtrInteger(42)
        self.assertEqual(tester.valueOfSharedPtrInteger(integerPtr), 42)

        strPtr = tester.createSharedPtrStr('hello')
        self.assertEqual(tester.valueOfSharedPtrStr(strPtr), 'hello')


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

"""
init_platform.py

Existence registry
==================

This is a registry for all existing function signatures.
One file is generated with all signatures of a platform and version.

The scope has been extended to generate all signatures from the
shiboken and pysidetest projects.
"""

import sys
import os
from contextlib import contextmanager
from textwrap import dedent
from util import get_refpath, get_script_dir


def qt_build():
    result = '<Unknown build of Qt>'
    try:
        from PySide6.QtCore import QLibraryInfo
        result = QLibraryInfo.build()
    except Exception:
        pass
    return result


script_dir = get_script_dir()
history_dir = os.path.join(script_dir, 'build_history')

# Find out if we have the build dir, already. Then use it.
look_for = os.path.join("pyside6", "tests", "pysidetest")
have_build_dir = [x for x in sys.path if x.endswith(look_for)]
if have_build_dir:
    all_build_dir = os.path.normpath(os.path.join(have_build_dir[0], "..", "..", ".."))
elif os.path.exists(history_dir):
    # Using the last build to find the build dir.
    # Note: This is not reliable when building in parallel!
    last_build = max(x for x in os.listdir(history_dir) if x.startswith("20"))
    fpath = os.path.join(history_dir, last_build, "build_dir.txt")
    if os.path.exists(fpath):
        with open(fpath) as f:
            f_contents = f.read().strip()
            f_contents_split = f_contents.splitlines()
            try:
                all_build_dir = f_contents_split[0]
            except IndexError:
                print(f"Error: can't find the build dir in the given file '{fpath}'")
                sys.exit(1)
else:
    print(dedent("""
        Can't find the build dir in the history.
        Compile again and don't forget to specify "--build-tests".
        """))
    sys.exit(1)

if not os.path.exists(os.path.join(all_build_dir, look_for)):
    print(dedent("""
        PySide has not been built with tests enabled.
        Compile again and don't forget to specify "--build-tests".
        """))
    sys.exit(1)

pyside_build_dir = os.path.join(all_build_dir, "pyside6")
shiboken_build_dir = os.path.join(all_build_dir, "shiboken6")

# now we compute all paths:


def set_ospaths(build_dir):
    ps = os.pathsep
    ospath_var = "PATH" if sys.platform == "win32" else "LD_LIBRARY_PATH"
    old_val = os.environ.get(ospath_var, "")
    lib_path = [os.path.join(build_dir, "pyside6", "tests", "pysidetest"),]
    ospath = ps.join(lib_path + old_val.split(ps))
    os.environ[ospath_var] = ospath


set_ospaths(all_build_dir)

import PySide6  # noqa: E402

all_modules = list("PySide6." + _ for _ in PySide6.__all__)

# now we should be able to do all imports:
if not have_build_dir:
    sys.path.insert(0, os.path.join(pyside_build_dir, "tests", "pysidetest"))
import testbinding  # noqa: E402 F401
all_modules.append("testbinding")

from shiboken6 import Shiboken  # noqa: E402 F401
all_modules.append("shiboken6.Shiboken")

from shibokensupport.signature.lib.enum_sig import SimplifyingEnumerator  # noqa: E402

# Make sure not to get .pyc in Python2.
sourcepath = os.path.splitext(__file__)[0] + ".py"


class Formatter:
    """
    Formatter is formatting the signature listing of an enumerator.

    It is written as context managers in order to avoid many callbacks.
    The separation in formatter and enumerator is done to keep the
    unrelated tasks of enumeration and formatting apart.
    """
    def __init__(self, outfile):
        self.outfile = outfile
        self.last_level = 0

    def print(self, *args, **kw):
        print(*args, file=self.outfile, **kw) if self.outfile else None

    @contextmanager
    def module(self, mod_name, *other):
        self.print()
        self.print(f"# Module {mod_name}")
        self.print("sig_dict.update({")
        yield
        self.print(f'    }}) if "{mod_name}" in sys.modules else None')

    @contextmanager
    def klass(self, class_name, class_str, *other):
        self.print()
        self.print(f"# class {self.mod_name}.{class_name}:")
        yield

    @contextmanager
    def function(self, func_name, signature, *other):
        if self.last_level > self.level:
            self.print()
        self.last_level = self.level
        class_name = self.class_name
        if class_name is None:
            key = viskey = f"{self.mod_name}.{func_name}"
        else:
            key = viskey = f"{self.mod_name}.{class_name}.{func_name}"
        if key.endswith("lY"):
            # Some classes like PySide6.QtGui.QContextMenuEvent have functions
            # globalX and the same with Y. The gerrit robot thinks that this
            # is a badly written "globally". Convince it by hiding this word.
            viskey = viskey[:-1] + '""Y'
        self.print(f'    "{viskey}": {signature},')
        yield key


def enum_all():
    fmt = Formatter(None)
    enu = SimplifyingEnumerator(fmt)
    ret = enu.result_type()
    for mod_name in all_modules:
        ret.update(enu.module(mod_name))
    return ret


LICENSE_TEXT = """
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations
"""


def generate_all():
    refPath = get_refpath()
    module = os.path.basename(os.path.splitext(refPath)[0])
    with open(refPath, "w") as outfile, open(sourcepath) as f:
        fmt = Formatter(outfile)
        enu = SimplifyingEnumerator(fmt)
        lines = f.readlines()
        license_line = next((lno for lno, line in enumerate(lines)  # noqa: F841
                             if "$QT_END_LICENSE$" in line))
        fmt.print("#recreate       # uncomment this to enforce generation")
        fmt.print(LICENSE_TEXT)
        version = sys.version.replace('\n', ' ')
        build = qt_build()
        fmt.print(dedent(f'''\
            """
            This file contains the simplified signatures for all functions in PySide
            for module '{module}' using
            Python {version}
            {build}

            There are no default values, no variable names and no self
            parameter. Only types are present after simplification.
            """
            '''))
        fmt.print("import sys")
        fmt.print("")
        fmt.print("sig_dict = {}")
        for mod_name in all_modules:
            enu.module(mod_name)
        fmt.print("# eof")


def __main__():
    print(f"+++ generating {get_refpath()}. You should probably check this file in.")
    generate_all()


if __name__ == "__main__":
    __main__()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the corelib/mimetypes/mimetypebrowser example from from Qt"""

import argparse
import sys

from mainwindow import MainWindow
from PySide6.QtCore import QLibraryInfo, QLocale, QTranslator
from PySide6.QtWidgets import QApplication

if __name__ == "__main__":
    app = QApplication(sys.argv)
    translator = QTranslator(app)
    if translator.load(QLocale.system(), "qtbase", "_",
                       QLibraryInfo.path(QLibraryInfo.LibraryPath.TranslationsPath)):
        app.installTranslator(translator)

    parser = argparse.ArgumentParser(description="MimeTypesBrowser Example")
    parser.add_argument("-v", "--version", action="version", version="%(prog)s 1.0")
    args = parser.parse_args()

    mainWindow = MainWindow()
    availableGeometry = mainWindow.screen().availableGeometry()
    mainWindow.resize(availableGeometry.width() / 3, availableGeometry.height() / 2)
    mainWindow.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import fnmatch
import os
import shutil
import zipfile
import sys
from argparse import ArgumentParser, RawTextHelpFormatter
from dataclasses import dataclass
from enum import IntEnum, Enum
from pathlib import Path
from collections import defaultdict
from typing import DefaultDict

sys.path.append(os.fspath(Path(__file__).parent.parent.parent / "sources" / "pyside-tools"))
from project_lib import parse_pyproject_json, parse_pyproject_toml, \
    PYPROJECT_FILE_PATTERNS, PYPROJECT_TOML_PATTERN, PYPROJECT_JSON_PATTERN  # noqa: E402


class Format(Enum):
    RST = 0
    MD = 1


__doc__ = """\
This tool scans the main repository for examples with project files and generates a documentation
page formatted as a gallery, displaying the examples in a table

For the usage, simply run:
    python tools/example_gallery/main.py
"""
DIR = Path(__file__).parent
EXAMPLES_DOC = Path(f"{DIR}/../../sources/pyside6/doc/examples").resolve()
EXAMPLES_DIR = Path(f"{DIR}/../../examples/").resolve()
TARGET_HELP = f"Directory into which to generate Doc files (default: {str(EXAMPLES_DOC)})"
BASE_URL = "https://code.qt.io/cgit/pyside/pyside-setup.git/tree"
DOC_SUFFIXES = {Format.RST: "rst", Format.MD: "md"}
LITERAL_INCLUDE = ".. literalinclude::"
IMAGE_SUFFIXES = (".png", ".jpg", ".jpeg", ".gif", ".svg", ".svgz", ".webp")
# Suffixes to ignore when displaying source files that are referenced in the project file
IGNORED_SUFFIXES = IMAGE_SUFFIXES + (".pdf", ".pyc", ".obj", ".mesh")
LANGUAGE_PATTERNS = {
    "*.h": "cpp",
    "*.cpp": "cpp",
    "*.md": "markdown",
    "*.py": "py",
    "*.qml": "js",
    "*.qmlproject": "js",
    "*.conf": "ini",
    "*.qrc": "xml",
    "*.ui": "xml",
    "*.xbel": "xml",
    "*.xml": "xml",
    "*.html": "html",
    "CMakeLists.txt": "cmake",
}

BASE_CONTENT = """\
.. _pyside6_examples:

Examples
========

 A collection of examples are provided with |project| to help new users
 to understand different use cases of the module.

 You can find all these examples inside the
 `pyside-setup <https://code.qt.io/cgit/pyside/pyside-setup.git/>`_ repository
 on the `examples <https://code.qt.io/cgit/pyside/pyside-setup.git/tree/examples>`_
 directory.

"""
# We generate a 'toctree' at the end of the file to include the new 'example' rst files, so we get
# no warnings and also that users looking for them will be able to, since they are indexed
# Notice that :hidden: will not add the list of files by the end of the main examples HTML page.
FOOTER_INDEX = """\
.. toctree::
    :hidden:
    :maxdepth: 1

"""
TUTORIAL_HEADLINES = {
    "tutorials/extending-qml/chapter": "Tutorial: Writing QML Extensions with Python",
    "tutorials/extending-qml-advanced/advanced": "Tutorial: Writing advanced QML Extensions with"
                                                 "Python",
    "tutorials/finance_manager": "Tutorial: Finance Manager - Integrating PySide6 with SQLAlchemy "
                                 "and FastAPI",
}


class ModuleType(IntEnum):
    ESSENTIALS = 0
    ADDONS = 1
    M2M = 2


def get_lexer(path: Path) -> str:
    """Given a file path, return the language lexer to use for syntax highlighting"""
    for pattern, lexer in LANGUAGE_PATTERNS.items():
        if fnmatch.fnmatch(path.name, pattern):
            return lexer
    # Default to text
    return "text"


def ind(level: int) -> str:
    """Return a string of spaces for string indentation given certain level"""
    return " " * 4 * level


def add_indent(s: str, level: int) -> str:
    """Add indentation to a string"""
    new_s = ""
    for line in s.splitlines():
        if line.strip():
            new_s += f"{ind(level)}{line}\n"
        else:
            # Empty line
            new_s += "\n"
    return new_s


def check_img_ext(image_path: Path) -> bool:
    """Check whether a file path is an image depending on its suffix."""
    return image_path.suffix in IMAGE_SUFFIXES


@dataclass
class ModuleDescription:
    """Specifies a sort key and type for a Qt module."""
    sort_key: int = 0
    module_type: ModuleType = ModuleType.ESSENTIALS
    description: str = ''


MODULE_DESCRIPTIONS = {
    "async": ModuleDescription(16, ModuleType.ESSENTIALS, ''),
    "corelib": ModuleDescription(15, ModuleType.ESSENTIALS, ''),
    "dbus": ModuleDescription(22, ModuleType.ESSENTIALS, ''),
    "designer": ModuleDescription(11, ModuleType.ESSENTIALS, ''),
    "gui": ModuleDescription(25, ModuleType.ESSENTIALS, ''),
    "network": ModuleDescription(20, ModuleType.ESSENTIALS, ''),
    "opengl": ModuleDescription(26, ModuleType.ESSENTIALS, ''),
    "qml": ModuleDescription(0, ModuleType.ESSENTIALS, ''),
    "quick": ModuleDescription(1, ModuleType.ESSENTIALS, ''),
    "quickcontrols": ModuleDescription(2, ModuleType.ESSENTIALS, ''),
    "samplebinding": ModuleDescription(30, ModuleType.ESSENTIALS, ''),
    "scriptableapplication": ModuleDescription(30, ModuleType.ESSENTIALS, ''),
    "sql": ModuleDescription(21, ModuleType.ESSENTIALS, ''),
    "uitools": ModuleDescription(12, ModuleType.ESSENTIALS, ''),
    "widgetbinding": ModuleDescription(30, ModuleType.ESSENTIALS, ''),
    "widgets": ModuleDescription(10, ModuleType.ESSENTIALS, ''),
    "xml": ModuleDescription(24, ModuleType.ESSENTIALS, ''),
    "Qt Demos": ModuleDescription(0, ModuleType.ADDONS, ''),  # from Qt repos
    "3d": ModuleDescription(30, ModuleType.ADDONS, ''),
    "axcontainer": ModuleDescription(20, ModuleType.ADDONS, ''),
    "bluetooth": ModuleDescription(20, ModuleType.ADDONS, ''),
    "charts": ModuleDescription(12, ModuleType.ADDONS, ''),
    "datavisualization": ModuleDescription(11, ModuleType.ADDONS, ''),
    "demos": ModuleDescription(0, ModuleType.ADDONS, ''),
    "external": ModuleDescription(20, ModuleType.ADDONS, ''),
    "graphs": ModuleDescription(10, ModuleType.ADDONS, ''),
    "httpserver": ModuleDescription(0, ModuleType.ADDONS, ''),
    "location": ModuleDescription(20, ModuleType.ADDONS, ''),
    "multimedia": ModuleDescription(12, ModuleType.ADDONS, ''),
    "networkauth": ModuleDescription(20, ModuleType.ADDONS, ''),
    "pdf": ModuleDescription(20, ModuleType.ADDONS, ''),
    "pdfwidgets": ModuleDescription(20, ModuleType.ADDONS, ''),
    "quick3d": ModuleDescription(20, ModuleType.ADDONS, ''),
    "remoteobjects": ModuleDescription(20, ModuleType.ADDONS, ''),
    "serialbus": ModuleDescription(30, ModuleType.ADDONS, ''),
    "serialport": ModuleDescription(30, ModuleType.ADDONS, ''),
    "spatialaudio": ModuleDescription(20, ModuleType.ADDONS, ''),
    "speech": ModuleDescription(20, ModuleType.ADDONS, ''),
    "statemachine": ModuleDescription(30, ModuleType.ADDONS, ''),
    "webchannel": ModuleDescription(30, ModuleType.ADDONS, ''),
    "webenginequick": ModuleDescription(15, ModuleType.ADDONS, ''),
    "webenginewidgets": ModuleDescription(16, ModuleType.ADDONS, ''),
    "coap": ModuleDescription(0, ModuleType.M2M, ''),
    "mqtt": ModuleDescription(0, ModuleType.M2M, ''),
    "opcua": ModuleDescription(0, ModuleType.M2M, '')
}


def module_sort_key(name: str) -> str:
    """Return a key for sorting the Qt modules."""
    description = MODULE_DESCRIPTIONS.get(name)
    module_type = int(description.module_type) if description else 5
    sort_key = description.sort_key if description else 100
    return f"{module_type}:{sort_key:04}:{name}"


def module_title(name: str) -> str:
    """Return title for a module."""
    result = name.title()
    description = MODULE_DESCRIPTIONS.get(name)
    if description:
        if description.description:
            result += " - " + description.description
        if description.module_type == ModuleType.M2M:
            result += " (M2M)"
        elif description.module_type == ModuleType.ADDONS:
            result += " (Add-ons)"
        else:
            result += " (Essentials)"
    return result


@dataclass
class ExampleData:
    """Example data for formatting the gallery."""

    example_name: str = None
    module: str = None
    extra: str = None
    doc_file: str = None
    file_format: Format = None
    abs_path: str = None
    src_doc_file: Path = None
    img_doc: Path = None
    tutorial: str = None
    headline: str = ""


def get_module_gallery(examples: list[ExampleData]) -> str:
    """
    This function takes a list of examples from one specific module and returns the resulting string
    in RST format that can be used to generate the table for the examples
    """

    gallery = (
        f"{ind(1)}.. grid:: 1 3 3 3\n"
        f"{ind(2)}:gutter: 3\n\n"
    )

    for i, example in enumerate(examples):
        suffix = DOC_SUFFIXES[example.file_format]
        # doc_file with suffix removed, to be used as a sphinx reference
        # lower case sphinx reference
        # this seems to be a bug or a requirement from sphinx
        doc_file_name = example.doc_file.replace(f".{suffix}", "").lower()

        name = example.example_name
        underline = example.module

        if example.extra:
            underline += f"/{example.extra}"

        if i > 0:
            gallery += "\n"
        img_name = example.img_doc.name if example.img_doc else "../example_no_image.png"

        # Fix long names
        if name.startswith("chapter"):
            name = name.replace("chapter", "c")
        elif name.startswith("advanced"):
            name = name.replace("advanced", "a")

        # Handling description from original file
        desc = ""
        if example.src_doc_file:
            with example.src_doc_file.open("r", encoding="utf-8") as f:
                # Read the first line
                first_line = f.readline().strip()

                # Check if the first line is a reference (starts with '(' and ends with ')=' for MD,
                # or starts with '.. ' and ends with '::' for RST)
                if ((example.file_format == Format.MD and first_line.startswith('(')
                     and first_line.endswith(')='))
                        or (example.file_format == Format.RST and first_line.startswith('.. ')
                            and first_line.endswith('::'))):
                    # The first line is a reference, so read the next lines until a non-empty line
                    # is found
                    while True:
                        title_line = f.readline().strip()
                        if title_line:
                            break
                else:
                    # The first line is the title
                    title_line = first_line

                # The next line handling depends on the file format
                line = f.readline().strip()

                if example.file_format == Format.MD:
                    # For markdown, the second line is the empty line
                    pass
                else:
                    # For RST and other formats, the second line is the underline under the title
                    # The next line should be empty
                    line = f.readline().strip()

                if line != "":
                    raise RuntimeError(
                        f"{line} was expected to be empty. Doc file: {example.src_doc_file}")

                # Now read until another empty line
                lines = []
                while True:
                    line = f.readline().strip()
                    if line.startswith(".. tags") or line.startswith("#"):
                        # Skip the empty line
                        _ = f.readline()
                        # Read the next line
                        line = f.readline().strip()

                    if not line:
                        break
                    lines.append(line)

                desc = " ".join(lines)
                if len(desc) > 120:
                    desc = desc[:120] + "..."
        else:
            if not opt_quiet:
                print(
                    f"example_gallery: No source doc file found in {example.abs_path}."
                    f"Skipping example",
                    file=sys.stderr,
                )

        if not (title := example.headline):
            title = f"{name} from ``{underline}``."

        # Clean refs from desc
        if ":ref:" in desc:
            desc = desc.replace(":ref:`", "")
        desc = desc.replace("`", "")

        gallery += f"{ind(2)}.. grid-item-card:: {title}\n"
        gallery += f"{ind(3)}:class-item: cover-img\n"
        gallery += f"{ind(3)}:link: {doc_file_name}\n"
        gallery += f"{ind(3)}:link-type: ref\n"
        gallery += f"{ind(3)}:img-top: {img_name}\n\n"
        gallery += f"{ind(3)}+++\n"
        gallery += f"{ind(3)}{desc}\n"

    return f"{gallery}\n"


def remove_licenses(file_content: str) -> str:
    # Return the content of the file with the Qt license removed
    new_s = []
    for line in file_content.splitlines():
        if line.strip().startswith(("/*", "**", "##")):
            continue
        new_s.append(line)
    return "\n".join(new_s)


def make_zip_archive(output_path: Path, src: Path, skip_dirs: list[str] = None):
    # Create a .zip file from a source directory ignoring the specified directories
    src_path = src.expanduser().resolve(strict=True)
    if skip_dirs is None:
        skip_dirs = []
    if not isinstance(skip_dirs, list):
        raise ValueError("Type error: 'skip_dirs' must be a list instance")
    with zipfile.ZipFile(output_path, 'w', zipfile.ZIP_DEFLATED) as zf:
        for file in src_path.rglob('*'):
            skip = False
            _parts = file.relative_to(src_path).parts
            for sd in skip_dirs:
                if sd in _parts:
                    skip = True
                    break
            if not skip:
                zf.write(file, file.relative_to(src_path.parent))


def get_rstinc_for_file(project_dir: Path, project_file: Path) -> Path | None:
    """Return the .rstinc file in the doc folder describing a source file, if found"""
    rst_file = project_dir
    if project_dir.name != "doc":  # Special case: Dummy .pyproject file in doc dir
        rst_file /= "doc"
    rst_file /= project_file.name + ".rstinc"
    return rst_file if rst_file.is_file() else None


def get_code_tabs(files: list[Path], project_dir: Path, file_format: Format) -> str:
    """
    Return the string which contains the code tabs source for the example
    Also creates a .zip file for downloading the source files
    """
    content = "\n"

    # Prepare ZIP file, and copy to final destination
    zip_name = f"{project_dir.name}.zip"
    make_zip_archive(EXAMPLES_DOC / zip_name, project_dir, skip_dirs=["doc"])

    if file_format == Format.RST:
        content += f":download:`Download this example <{zip_name}>`\n\n"
    elif file_format == Format.MD:
        content += f"{{download}}`Download this example <{zip_name}>`\n\n"
        # MD files wrap the content in a eval-rst block
        content += "```{eval-rst}\n"
    else:
        raise ValueError(f"Unknown documentation file format {file_format}")

    if files:
        content += ".. tab-set::\n\n"

    for i, file in enumerate(files):
        if file.suffix in IGNORED_SUFFIXES:
            continue

        try:
            tab_title = file.relative_to(project_dir).as_posix()
        except ValueError:
            # The file is outside project_dir, so the best we can do is to use the file name
            tab_title = file.name

        content += f"{ind(1)}.. tab-item:: {tab_title}\n\n"

        if doc_rstinc_file := get_rstinc_for_file(project_dir, file):
            content += add_indent(doc_rstinc_file.read_text("utf-8"), 2)
            content += "\n"

        content += add_indent(f"{ind(1)}.. code-block:: {get_lexer(file)}", 1)
        content += "\n"

        try:
            file_content = remove_licenses(file.read_text(encoding="utf-8"))
        except UnicodeDecodeError as e:
            raise RuntimeError(f"example_gallery: error decoding {file}: {e}")
        except FileNotFoundError as e:
            raise RuntimeError(f"example_gallery: error opening {file}: {e}")

        content += add_indent(file_content, 3)
        content += "\n\n"

    if file_format == Format.MD:
        # Close the eval-rst block
        content += "```"

    return content


def get_default_header_title(example_dir: Path) -> str:
    """Get a default header title for an example directory without a doc file"""
    _index = example_dir.parts.index("examples")
    rel_path = "/".join(example_dir.parts[_index:])
    _title = rel_path
    url = f"{BASE_URL}/{rel_path}"
    return (
        "..\n    This file was auto-generated by the 'examples_gallery' "
        "script.\n    Any change will be lost!\n\n"
        f"{_title}\n"
        f"{'=' * len(_title)}\n\n"
        f"(You can also check this code `in the repository <{url}>`_)\n\n"
    )


def rel_path(from_path: Path, to_path: Path) -> str:
    """
    Get a relative path for a given path that is not a subpath (where Path.relative_to() fails)
    of from_path via a common ancestor path

    For example: from_path = Path("/a/b/c/d"), to_path = Path("/a/b/e/f"). Returns: "../../e/f"
    """
    common_path = Path(*os.path.commonprefix([from_path.parts, to_path.parts]))
    up_dirs = len(from_path.parts) - len(common_path.parts)
    prefix = up_dirs * "../"
    relative_to_common = to_path.relative_to(common_path).as_posix()
    return f"{prefix}{relative_to_common}"


def read_rst_file(project_dir: Path, project_files: list[Path], doc_rst: Path) -> str:
    """
    Read the example .rst file and replace Sphinx literal includes of project files by paths
    relative to the example directory
    Note: Sphinx does not handle absolute paths as expected, they need to be relative
    """
    content = Path(doc_rst).read_text(encoding="utf-8")
    if LITERAL_INCLUDE not in content:
        # The file does not contain any literal includes, so we can return it as is
        return content

    result = []
    path_to_example = rel_path(EXAMPLES_DOC, project_dir)
    for line in content.split("\n"):
        if line.startswith(LITERAL_INCLUDE):
            file = line[len(LITERAL_INCLUDE) + 1:].strip()
            file_path = project_dir / file
            if file_path not in project_files:
                raise RuntimeError(f"File {file} not found in project files: {project_files}")
            line = f"{LITERAL_INCLUDE} {path_to_example}/{file}"
        result.append(line)
    return "\n".join(result)


def get_headline(text: str, file_format: Format) -> str:
    """Find the headline in the documentation file."""
    if file_format == Format.RST:
        underline = text.find("\n====")
        if underline != -1:
            start = text.rfind("\n", 0, underline - 1)
            return text[start + 1:underline]
    elif file_format == Format.MD:
        headline = text.find("# ")
        if headline != -1:
            new_line = text.find("\n", headline + 1)
            if new_line != -1:
                return text[headline + 2:new_line].strip()
    else:
        raise ValueError(f"Unknown file format {file_format}")
    return ""


def get_doc_source_file(original_doc_dir: Path, example_name: str) -> tuple[Path, Format] | None:
    """
    Find the doc source file given a doc directory and an example name
    Returns the doc file path and the file format, if found
    """
    if not original_doc_dir.is_dir():
        return None

    for file_format, suffix in DOC_SUFFIXES.items():
        result = original_doc_dir / f"{example_name}.{suffix}"
        if result.is_file():
            return result, file_format
    return None


def get_screenshot(image_dir: Path, example_name: str) -> Path | None:
    """
    Find an screenshot given an image directory and the example name
    Returns the image with the same example_name, if found
    If not found, the first image in the directory is returned
    """
    if not image_dir.is_dir():
        return None
    images = [i for i in image_dir.glob("*") if i.is_file() and check_img_ext(i)]
    example_images = [i for i in images if i.name.startswith(example_name)]
    if example_images:
        return example_images[0]
    if images:
        return images[0]
    return None


def write_resources(src_list: list[Path], dst: Path):
    """Write a list of example resource paths to the dst path."""
    for src in src_list:
        resource_written = shutil.copy(src, dst / src.name)
        if not opt_quiet:
            print(f"Written resource: {resource_written}")


@dataclass
class ExampleParameters:
    """Parameters obtained from scanning the examples directory."""
    example_dir: Path = None
    module_name: str = ""
    example_name: str = ""
    target_doc_file: str = None
    extra_names: str = ""
    src_doc_dir: Path = None
    src_doc_file_path: Path = None
    src_screenshot: Path = None
    file_format: Format = Format.RST


def get_pyside_example_parameters(example_root: Path, pyproject_file: Path) -> ExampleParameters:
    """Analyze a PySide example folder to get the example parameters"""
    p = ExampleParameters()

    p.example_dir = pyproject_file.parent
    if list(p.example_dir.parent.glob("*.qmlproject")) and p.example_dir.name == "Python":
        # Design Studio project example
        p.example_dir = pyproject_file.parent.parent

    if p.example_dir.name == "doc":  # Dummy pyproject file in doc dir (e.g. scriptableapplication)
        p.example_dir = p.example_dir.parent

    parts = p.example_dir.parts[len(example_root.parts):]
    p.module_name = parts[0]
    p.example_name = parts[-1]
    # handling subdirectories besides the module level and the example
    p.extra_names = "" if len(parts) == 2 else "_".join(parts[1:-1])

    # Check for a 'doc' directory inside the example
    src_doc_dir = p.example_dir / "doc"

    if src_doc_dir.is_dir():
        src_doc_file = get_doc_source_file(src_doc_dir, p.example_name)
        if src_doc_file:
            p.src_doc_file_path, p.file_format = src_doc_file
            p.src_doc_dir = src_doc_dir
            p.src_screenshot = get_screenshot(src_doc_dir, p.example_name)

    target_suffix = DOC_SUFFIXES[p.file_format]
    doc_file = f"example_{p.module_name}_{p.extra_names}_{p.example_name}.{target_suffix}"
    p.target_doc_file = doc_file.replace("__", "_")
    return p


def get_qt_example_parameters(pyproject_file: Path) -> ExampleParameters:
    """
    Analyze a Qt repository example to get its parameters.
    For instance, the qtdoc/examples/demos/mediaplayer example
    """
    p = ExampleParameters()

    p.example_dir = pyproject_file.parent
    p.module_name = "Qt Demos"
    p.example_name = p.example_dir.name
    # Check for a 'doc' directory inside the example (qdoc)
    doc_root = p.example_dir / "doc"
    if doc_root.is_dir():
        src_doc_file = get_doc_source_file(doc_root / "src", p.example_name)
        if src_doc_file:
            p.src_doc_file_path, p.file_format = src_doc_file
            p.src_doc_dir = doc_root
            p.src_screenshot = get_screenshot(doc_root / "images", p.example_name)
        else:
            raise ValueError(f"No source file found in {doc_root} / src given {p.example_name}")
    else:
        raise ValueError(f"No doc directory found in {p.example_dir}")
    target_suffix = DOC_SUFFIXES[p.file_format]
    p.target_doc_file = f"example_qtdemos_{p.example_name}.{target_suffix}"
    return p


def write_example(
    example_root: Path, pyproject_file: Path, pyside_example: bool = True
) -> tuple[str, ExampleData]:
    """
    Read the project file and documentation, create the .rst file and copy the example data
    Return a tuple with the module name and an ExampleData instance
    """
    # Get the example parameters depending on whether it is a PySide example or a Qt one
    p: ExampleParameters = (
        get_pyside_example_parameters(example_root, pyproject_file)
        if pyside_example else get_qt_example_parameters(pyproject_file))

    result = ExampleData()
    result.example_name = p.example_name
    result.module = p.module_name
    result.extra = p.extra_names
    result.doc_file = p.target_doc_file
    result.file_format = p.file_format
    result.abs_path = str(p.example_dir)
    result.src_doc_file = p.src_doc_file_path
    result.img_doc = p.src_screenshot
    result.tutorial = TUTORIAL_HEADLINES.get(result.abs_path, "")

    if pyproject_file.match(PYPROJECT_JSON_PATTERN):
        pyproject_parse_result = parse_pyproject_json(pyproject_file)
    elif pyproject_file.match(PYPROJECT_TOML_PATTERN):
        pyproject_parse_result = parse_pyproject_toml(pyproject_file)
    else:
        raise RuntimeError(f"Invalid project file: {pyproject_file}")

    if pyproject_parse_result.errors:
        raise RuntimeError(f"Error reading {pyproject_file}: {pyproject_parse_result.errors}")

    for file in pyproject_parse_result.files:
        if not Path(file).exists:
            raise FileNotFoundError(f"{file} listed in {pyproject_file} does not exist")

    files = pyproject_parse_result.files
    headline = ""
    if files:
        doc_file = EXAMPLES_DOC / p.target_doc_file
        sphnx_ref_example = p.target_doc_file.replace(f'.{DOC_SUFFIXES[p.file_format]}', '')
        # lower case sphinx reference
        # this seems to be a bug or a requirement from sphinx
        sphnx_ref_example = sphnx_ref_example.lower()

        if p.file_format == Format.RST:
            content_f = f".. _{sphnx_ref_example}:\n\n"
        elif p.file_format == Format.MD:
            content_f = f"({sphnx_ref_example})=\n\n"
        else:
            raise ValueError(f"Invalid file format {p.file_format}")

        with open(doc_file, "w", encoding="utf-8") as out_f:
            if p.src_doc_file_path:
                content_f += read_rst_file(p.example_dir, files, p.src_doc_file_path)
                headline = get_headline(content_f, p.file_format)
                if not headline and not opt_quiet:
                    print(f"example_gallery: No headline found in {doc_file}", file=sys.stderr)

                # Copy other files in the 'doc' directory, but excluding the main '.rst' file and
                # all the directories
                resources = []
                if pyside_example:
                    for _f in p.src_doc_dir.glob("*"):
                        if _f != p.src_doc_file_path and not _f.is_dir():
                            resources.append(_f)
                elif p.src_screenshot:
                    # Qt example: only use image, if found
                    resources.append(p.src_screenshot)
                write_resources(resources, EXAMPLES_DOC)
            else:
                content_f += get_default_header_title(p.example_dir)
            content_f += get_code_tabs(files, p.example_dir, p.file_format)
            out_f.write(content_f)

        if not opt_quiet:
            print(f"Written: {doc_file}")
    else:
        if not opt_quiet:
            print(f"{pyproject_file} does not contain any file, skipping")

    result.headline = headline

    return p.module_name, result


def example_sort_key(example: ExampleData) -> str:
    """
    Return a key for sorting the examples. Tutorials are sorted first, then the examples which
    contain "gallery" in their name, then alphabetically
    """
    result = ""
    if example.tutorial:
        result += "AA:" + example.tutorial + ":"
    elif "gallery" in example.example_name:
        result += "AB:"
    result += example.example_name
    return result


def sort_examples(examples: dict[str, list[ExampleData]]) -> dict[str, list[ExampleData]]:
    """Sort the examples using a custom function."""
    result = {}
    for module in examples.keys():
        result[module] = sorted(examples.get(module), key=example_sort_key)
    return result


def scan_examples_dir(
    examples_dir: Path, pyside_example: bool = True
) -> dict[str, list[ExampleData]]:
    """
    Scan a directory of examples and return a dictionary with the found examples grouped by module
    Also creates the .rst file for each example
    """
    examples: dict[str, list[ExampleData]] = defaultdict(list)
    # Find all the project files contained in the examples directory
    project_files: list[Path] = []
    for project_file_pattern in PYPROJECT_FILE_PATTERNS:
        project_files.extend(examples_dir.glob(f"**/{project_file_pattern}"))

    for project_file in project_files:
        if project_file.name == "examples.pyproject":
            # Ignore this project file. It contains files from many examples
            continue

        module_name, data = write_example(examples_dir, project_file, pyside_example)
        examples[module_name].append(data)

    return dict(examples)


if __name__ == "__main__":
    parser = ArgumentParser(description=__doc__, formatter_class=RawTextHelpFormatter)
    parser.add_argument("--target", "-t", action="store", dest="target_dir", help=TARGET_HELP)
    parser.add_argument("--qt-src-dir", "-s", action="store", help="Qt source directory")
    parser.add_argument("--quiet", "-q", action="store_true", help="Quiet")
    options = parser.parse_args()
    opt_quiet = options.quiet
    if options.target_dir:
        EXAMPLES_DOC = Path(options.target_dir).resolve()

    # This script will be in charge of:
    #   * Getting all the project files
    #   * Gather the information of the examples
    #   * Read the project file to output the content of each source file
    #     on the final .rst file for that specific example

    # Create the 'examples' directory if it doesn't exist
    # If it does exist, try to remove it and create a new one to start fresh
    if EXAMPLES_DOC.is_dir():
        shutil.rmtree(EXAMPLES_DOC, ignore_errors=True)
        if not opt_quiet:
            print("WARNING: Deleted existing examples HTML directory")
    EXAMPLES_DOC.mkdir(exist_ok=True)

    examples = scan_examples_dir(EXAMPLES_DIR)

    if options.qt_src_dir:
        # Scan the Qt source directory for Qt examples and include them in the dictionary of
        # discovered examples
        qt_src = Path(options.qt_src_dir)
        if not qt_src.is_dir():
            raise RuntimeError(f"Invalid Qt source directory: {qt_src}")
        examples.update(scan_examples_dir(qt_src.parent / "qtdoc", pyside_example=False))

    examples = sort_examples(examples)

    # List of all the example files found to be included in the index table of contents
    index_files: list[str] = []
    # Write the main example .rst file and the example files
    with open(f"{EXAMPLES_DOC}/index.rst", "w") as f:
        f.write(BASE_CONTENT)
        for module_name in sorted(examples.keys(), key=module_sort_key):
            module_examples = examples.get(module_name)
            tutorial_examples: DefaultDict[str, list[ExampleData]] = defaultdict(list)
            non_tutorial_examples: list[ExampleData] = []

            for example in module_examples:
                index_files.append(example.doc_file)
                if example.tutorial:
                    tutorial_examples[example.tutorial].append(example)
                else:
                    non_tutorial_examples.append(example)

            title = module_title(module_name)
            f.write(f".. dropdown:: {title}\n\n")

            # Write tutorial examples under their tutorial names
            for tutorial_name, tutorial_exs in tutorial_examples.items():
                f.write(f"{ind(1)}**{tutorial_name}**\n\n")
                f.write(get_module_gallery(tutorial_exs))

            # If there are non-tutorial examples and tutorials exist
            if tutorial_examples and non_tutorial_examples:
                f.write(f"{ind(1)}**Other Examples**\n\n")
                f.write(get_module_gallery(non_tutorial_examples))
            # If no tutorials exist, list all examples
            elif not tutorial_examples:
                f.write(get_module_gallery(module_examples))

        f.write("\n\n")

        # Add the list of the example files found to the index table of contents
        f.write(FOOTER_INDEX)
        for index_file in index_files:
            f.write(f"{ind(1)}{index_file}\n")

    if not opt_quiet:
        print(f"Written index: {EXAMPLES_DOC}/index.rst")



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit tests for QUuid'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QUuid


class QUuidTest(unittest.TestCase):
    def testFromString(self):
        uuidString = '{fc69b59e-cc34-4436-a43c-ee95d128b8c5}'
#       testing overload QUUid::fromString(QStringView)
        uuid = QUuid.fromString(uuidString)
        self.assertTrue(not uuid.isNull())
        self.assertEqual(uuid.toString(), uuidString)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2011 Arun Srinivasan <rulfzid@gmail.com>
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

try:
    import cpickle as pickle
except ImportError:
    import pickle

from PySide6.QtCore import (Qt, Signal, Slot, QRegularExpression, QModelIndex,
                            QItemSelection, QSortFilterProxyModel)
from PySide6.QtWidgets import QTabWidget, QMessageBox, QTableView, QAbstractItemView

from tablemodel import TableModel
from newaddresstab import NewAddressTab
from adddialogwidget import AddDialogWidget


class AddressWidget(QTabWidget):
    """ The central widget of the application. Most of the addressbook's
        functionality is contained in this class.
    """

    selection_changed = Signal(QItemSelection)

    def __init__(self, parent=None):
        """ Initialize the AddressWidget. """
        super().__init__(parent)

        self._table_model = TableModel()
        self._new_address_tab = NewAddressTab()
        self._new_address_tab.send_details.connect(self.add_entry)

        self.addTab(self._new_address_tab, "Address Book")

        self.setup_tabs()

    @Slot()
    def add_entry(self, name=None, address=None):
        """ Add an entry to the addressbook. """
        if name is None and address is None:
            add_dialog = AddDialogWidget()

            if add_dialog.exec():
                name = add_dialog.name
                address = add_dialog.address

        address = {"name": name, "address": address}
        addresses = self._table_model.addresses[:]

        # The QT docs for this example state that what we're doing here
        # is checking if the entered name already exists. What they
        # (and we here) are actually doing is checking if the whole
        # name/address pair exists already - ok for the purposes of this
        # example, but obviously not how a real addressbook application
        # should behave.
        try:
            addresses.remove(address)
            QMessageBox.information(self, "Duplicate Name",
                                    f'The name "{name}" already exists.')
        except ValueError:
            # The address didn't already exist, so let's add it to the model.

            # Step 1: create the  row
            self._table_model.insertRows(0)

            # Step 2: get the index of the newly created row and use it.
            # to set the name
            ix = self._table_model.index(0, 0, QModelIndex())
            self._table_model.setData(ix, address["name"], Qt.ItemDataRole.EditRole)

            # Step 3: lather, rinse, repeat for the address.
            ix = self._table_model.index(0, 1, QModelIndex())
            self._table_model.setData(ix, address["address"], Qt.ItemDataRole.EditRole)

            # Remove the newAddressTab, as we now have at least one
            # address in the model.
            self.removeTab(self.indexOf(self._new_address_tab))

            # The screenshot for the QT example shows nicely formatted
            # multiline cells, but the actual application doesn't behave
            # quite so nicely, at least on Ubuntu. Here we resize the newly
            # created row so that multiline addresses look reasonable.
            table_view = self.currentWidget()
            table_view.resizeRowToContents(ix.row())

    @Slot()
    def edit_entry(self):
        """ Edit an entry in the addressbook. """
        table_view = self.currentWidget()
        proxy_model = table_view.model()
        selection_model = table_view.selectionModel()

        # Get the name and address of the currently selected row.
        indexes = selection_model.selectedRows()
        if len(indexes) != 1:
            return

        row = proxy_model.mapToSource(indexes[0]).row()
        ix = self._table_model.index(row, 0, QModelIndex())
        name = self._table_model.data(ix, Qt.ItemDataRole.DisplayRole)
        ix = self._table_model.index(row, 1, QModelIndex())
        address = self._table_model.data(ix, Qt.ItemDataRole.DisplayRole)

        # Open an addDialogWidget, and only allow the user to edit the address.
        add_dialog = AddDialogWidget()
        add_dialog.setWindowTitle("Edit a Contact")

        add_dialog._name_text.setReadOnly(True)
        add_dialog._name_text.setText(name)
        add_dialog._address_text.setText(address)

        # If the address is different, add it to the model.
        if add_dialog.exec():
            new_address = add_dialog.address
            if new_address != address:
                ix = self._table_model.index(row, 1, QModelIndex())
                self._table_model.setData(ix, new_address, Qt.ItemDataRole.EditRole)

    @Slot()
    def remove_entry(self):
        """ Remove an entry from the addressbook. """
        table_view = self.currentWidget()
        proxy_model = table_view.model()
        selection_model = table_view.selectionModel()

        # Just like editEntry, but this time remove the selected row.
        indexes = selection_model.selectedRows()

        for index in indexes:
            row = proxy_model.mapToSource(index).row()
            self._table_model.removeRows(row)

        # If we've removed the last address in the model, display the
        # newAddressTab
        if self._table_model.rowCount() == 0:
            self.insertTab(0, self._new_address_tab, "Address Book")

    def setup_tabs(self):
        """ Setup the various tabs in the AddressWidget. """
        groups = ["ABC", "DEF", "GHI", "JKL", "MNO", "PQR", "STU", "VW", "XYZ"]

        for group in groups:
            proxy_model = QSortFilterProxyModel(self)
            proxy_model.setSourceModel(self._table_model)
            proxy_model.setDynamicSortFilter(True)

            table_view = QTableView()
            table_view.setModel(proxy_model)
            table_view.setSortingEnabled(True)
            table_view.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
            table_view.horizontalHeader().setStretchLastSection(True)
            table_view.verticalHeader().hide()
            table_view.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
            table_view.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)

            # This here be the magic: we use the group name (e.g. "ABC") to
            # build the regex for the QSortFilterProxyModel for the group's
            # tab. The regex will end up looking like "^[ABC].*", only
            # allowing this tab to display items where the name starts with
            # "A", "B", or "C". Notice that we set it to be case-insensitive.
            re = QRegularExpression(f"^[{group}].*")
            assert re.isValid()
            re.setPatternOptions(QRegularExpression.PatternOption.CaseInsensitiveOption)
            proxy_model.setFilterRegularExpression(re)
            proxy_model.setFilterKeyColumn(0)  # Filter on the "name" column
            proxy_model.sort(0, Qt.SortOrder.AscendingOrder)

            # This prevents an application crash (see:
            # https://www.qtcentre.org/threads/58874-QListView-SelectionModel-selectionChanged-Crash)  # noqa: E501
            self.viewselectionmodel = table_view.selectionModel()
            table_view.selectionModel().selectionChanged.connect(self.selection_changed)

            self.addTab(table_view, group)

    # Note: the QT example uses a QDataStream for the saving and loading.
    # Here we're using a python dictionary to store the addresses, which
    # can't be streamed using QDataStream, so we just use cpickle for this
    # example.
    def read_from_file(self, filename):
        """ Read contacts in from a file. """
        try:
            f = open(filename, "rb")
            addresses = pickle.load(f)
        except IOError:
            QMessageBox.information(self, f"Unable to open file: {filename}")
        finally:
            f.close()

        if len(addresses) == 0:
            QMessageBox.information(self, f"No contacts in file: {filename}")
        else:
            for address in addresses:
                self.add_entry(address["name"], address["address"])

    def write_to_file(self, filename):
        """ Save all contacts in the model to a file. """
        try:
            f = open(filename, "wb")
            pickle.dump(self._table_model.addresses, f)

        except IOError:
            QMessageBox.information(self, f"Unable to open file: {filename}")
        finally:
            f.close()


if __name__ == "__main__":
    import sys
    from PySide6.QtWidgets import QApplication

    app = QApplication(sys.argv)
    address_widget = AddressWidget()
    address_widget.show()
    sys.exit(app.exec())
